#include "curses.h"
#include <utility>
//#include <stdlib.h>
#include <time.h>
#include <random>
#include <vector>
#include <math.h>
#include <algorithm>
#include <fstream>
#include <string>
#include <iostream>

// TODO
//make bosses camp if they find a location they can damage from,
//and run if the player attacks them from a location they can't attack
//make map edges obviously walls

//? for help
//tutorial:
//  Hand-engineered weapons and maps to provide tactical situations,
//  so that how the patterns relate to the map can be demonstrated.
//  1. weapon with 1 damage and no restrictions, 5 HP, kill 10 'b's with the same weapon and 1 HP
//
//character dumps
//
//game modes:
//  unlimited (random enemy weapons, random floor weapons, no quality considerations, decreasing maxHP?)
//  

//these are from pdcurses
//#define KEY_A1        0x1c1  /* upper left on Virtual keypad */
#define KEY_A2        0x1c2  /* upper middle on Virt. keypad */
//#define KEY_A3        0x1c3  /* upper right on Vir. keypad */
#define KEY_B1        0x1c4  /* middle left on Virt. keypad */
//#define KEY_B2        0x1c5  /* center on Virt. keypad */
#define KEY_B3        0x1c6  /* middle right on Vir. keypad */
//#define KEY_C1        0x1c7  /* lower left on Virt. keypad */
#define KEY_C2        0x1c8  /* lower middle on Virt. keypad */
//#define KEY_C3        0x1c9  /* lower right on Vir. keypad */

//#define DEBUG_NO_DAMAGE 1

//[15:43:57] * srd|_| starts making encircled: the roguelike
//[03:35:27] <srd|> ok, 60 hours after starting Encircled is complete
// (A bunch of improvements were made after that, but the main engine was done and playable in 60 hours)
// This code is public domain. Do anything you want with it. If you have fun or do something interesting with it, email quendus@gmail.com .

//#ifdef LINUX
//#include <unistd.h>
//#endif
//#ifdef WINDOWS
////#include <windows.h>
//#include <dos.h>
//#endif

void mySleep(int sleepMs)
{
//#ifdef LINUX
//    usleep(sleepMs * 1000);   // usleep takes sleep time in us
//#endif
//#ifdef WINDOWS
//    Sleep(sleepMs);
	delay_output(sleepMs);
//#endif
}

bool nonmelee_available;
bool hurt_this_turn;
bool regen_this_turn;
int animation_delay = 200;
const int max_range = 8;
char game_mode = 0;
int dungeon_level;
int bosses_generated;
int boss_quota;
int bosses_killed;
int last_level;
int enemies_killed[5];
int damage_from_enemies[5];
int player_health_max(int dl)
{
	switch(game_mode)
	{
	case 's': case 'm':
		return 20+dl*5;
	case 'b': case 'q':
		return 20+dl*8;
	case 't':
		return dl==2?15:5;
	case 'u':
		return std::max(1,120-dl*5);
	}
	return 2;
}
int monster_health(int e)
{
	switch(game_mode)
	{
	case 's': case 'm':
		switch(e)
		{
		case 0: return 2*dungeon_level;
		case 1: return 4*dungeon_level;
		case 2: return 4*dungeon_level;
		case 3: return 4*dungeon_level;
		case 4: return dungeon_level*dungeon_level;
		}
		break;
	case 'q':
		switch(e)
		{
		case 0: return 3*dungeon_level;
		case 1: return 6*dungeon_level;
		case 2: return 6*dungeon_level;
		case 3: return 6*dungeon_level;
		case 4: return 3*dungeon_level*dungeon_level;
		}
		break;
	case 'b':
		switch(e)
		{
		case 0: return 4*dungeon_level;
		case 1: return 8*dungeon_level;
		case 2: return 8*dungeon_level;
		case 3: return 8*dungeon_level;
		case 4: return 4*dungeon_level*dungeon_level;
		}
		break;
	case 'u':
		switch(e)
		{
		case 0: return std::min(9999,2*dungeon_level);
		case 1: return std::min(9999,4*dungeon_level);
		case 2: return std::min(9999,4*dungeon_level);
		case 3: return std::min(9999,4*dungeon_level);
		case 4: return std::min(9999,dungeon_level*dungeon_level);
		}
		break;
	case 't':
		switch(e)
		{
		case 0: break;
		case 1: return 1; break;
		case 2: break;
		case 3: break;
		case 4: return 100;
			break;
		}
		break;
	}
	return 1;
}
int dungeon_level_number(int dl)
{
	switch(game_mode)
	{
	case 's': case 'm':
		if(dl<11)
			return dl;
		if(dl==12)
			return 11;
		if(dl==14)
			return 12;
		if(dl==15)
			return 13;
		if(dl==16)
			return 14;
		if(dl==18)
			return 15;
		if(dl==20)
			return 16;
		return 20;
		break;
	case 'q': case 't': case 'u': case 'b':
		return dl;
	}
	return dl;
}

namespace patterns
{
	struct weapon;
	std::vector<weapon*> weapons;
	int monster_weapon_id(int enemy_code)
	{
		switch(game_mode)
		{
		case 's': case 'm':
			return dungeon_level*(enemy_code+1) - 1;
		case 'q': case 'b':
			return dungeon_level*2*(enemy_code+1)-1;
		case 'u':
			return enemy_code+2;
		case 't':
			if(enemy_code==1)
			{
				if(dungeon_level == 1)
				{
					return 1;
				}
				else
				{
					return 5;
				}
			}
			else if(enemy_code==4)
			{
				return 4;
			}
		}
		return 0;
	}
}
void draw_ui();
void draw_dungeon();
void flash_number(int value,int x,int y,int colour,int source_x,int source_y);

class actor
{
public:
	int x, y;
	char ch;
	int col;
	bool is_player;
	int target_x;
	int target_y;
	bool go_for_restructuring;
	bool go_for_damage;
	bool dont_want_to_move;
	int help_this_enemy_type;
	int health;
	int max_health;
	patterns::weapon* my_weapon;
	patterns::weapon* my_other_weapon;
	actor(int xx, int yy, char cch, int ccol, int h, bool player=false)
	{
		x = xx;
		y = yy;
		ch = cch;
		col = ccol;
		is_player = player;
		health = h;
		max_health = h;
		go_for_restructuring=false;
		go_for_damage=false;
		
	}
};

actor* p_ptr;
std::vector<actor*> actors;

namespace patterns
{
void dungeon_restructure_real(actor* a, int count);
}


namespace terrains
{
	enum terrain {HELP=-3, SHOP=-2, WALL=-1, FLOOR = 0, RESTRUCTURE_10 = 1, RESTRUCTURE_20=2,
		RESTRUCTURE_30=3,RESTRUCTURE_40=4,RESTRUCTURE_50=5,RESTRUCTURE_60=6,
		RESTRUCTURE_70=7,RESTRUCTURE_80=8, RESTRUCTURE_90= 9};
	struct terrain_type
	{
		terrain t;
		bool permanent;
	};
	terrain_type tt(terrain t,bool p)
	{
		terrain_type ttt;
		ttt.t = t;
		ttt.permanent = p;
		return ttt;
	}
	enum terrain_class {CFLOOR = 0, CWALL = -1, CANY = 1};
	bool terrain_matches_class(terrain_class c, terrain_type t)
	{
		if(c == CANY) return true;
		if(c == CWALL) return t.t == WALL;
		if(c == CFLOOR) return t.t != WALL;
		return false;
	}
	char terrain_char(terrain t)
	{
		switch(t)
		{
		case HELP:
			return '?';
		case FLOOR:
			return '.';
		case WALL:
			return '#';
		case RESTRUCTURE_10:
			return '1';
		case RESTRUCTURE_20:
			return '2';
		case RESTRUCTURE_30:
			return '3';
		case RESTRUCTURE_40:
			return '4';
		case RESTRUCTURE_50:
			return '5';
		case RESTRUCTURE_60:
			return '6';
		case RESTRUCTURE_70:
			return '7';
		case RESTRUCTURE_80:
			return '8';
		case RESTRUCTURE_90:
			return '9';
		case SHOP:
			return '%';
		}
		return ' ';
	}
	char terrain_class_char(terrain_class t)
	{
		switch(t)
		{
		case CANY:
			return ' ';
		case CFLOOR:
			return '.';
		case CWALL:
			return '#';
		}
		return ' ';
	}
	
}

namespace dungeon
{
	const int MAP_WIDTH=50;
	const int MAP_HEIGHT=30;
	terrains::terrain_type dungeon_terrain[MAP_WIDTH*MAP_HEIGHT];
	bool enemies_can_attack[5][MAP_WIDTH*MAP_HEIGHT];
	bool enemies_can_attack_from[5][MAP_WIDTH*MAP_HEIGHT];
	bool player_can_attack[MAP_WIDTH*MAP_HEIGHT];
	bool player_can_attack_from[MAP_WIDTH*MAP_HEIGHT];
	bool player_can_attack_other[MAP_WIDTH*MAP_HEIGHT];
	bool player_can_attack_from_other[MAP_WIDTH*MAP_HEIGHT];
	
	int generator_responsible[MAP_WIDTH*MAP_HEIGHT];
	actor* dungeon_occupants[MAP_WIDTH*MAP_HEIGHT];
	patterns::weapon* dungeon_weapons[MAP_HEIGHT*MAP_WIDTH];
	int distance_to_player[MAP_HEIGHT*MAP_WIDTH];
	int dungeon_index(int x, int y)
	{
		return MAP_WIDTH*y+x;
	}
	bool on_map(int x, int y)
	{
		if (x<0 || y<0) return false;
		if(x >= MAP_WIDTH || y >= MAP_HEIGHT) return false;
		return true;
	}
	bool walkable(int x,int y)
	{
		if (!on_map(x,y))
			return false;
		terrains::terrain_type t = dungeon_terrain[dungeon_index(x,y)];
		if(t.t == terrains::WALL)
			return false;
		return true;
	}
	bool change_terrain(int x,int y,terrains::terrain_type t)
	{
		if(!on_map(x,y))
			return false;
		if(dungeon_terrain[dungeon_index(x,y)].permanent == true)
			return false;
		if(dungeon_occupants[dungeon_index(x,y)] != NULL)
			return false;
		dungeon_terrain[dungeon_index(x,y)]= t;
		return true;
	}
	int num_bosses_existing()
	{
		int count = 0;
		for (unsigned int i=0; i<actors.size(); i++)
		{
			if(actors[i]->ch == 'E' && actors[i]->health > 0)
				count++;
		}
		return count;
	}
	int num_bs_existing()
	{
		int count = 0;
		for (unsigned int i=0; i<actors.size(); i++)
		{
			if(actors[i]->ch == 'b' && actors[i]->health > 0)
				count++;
		}
		return count;
	}

	void add_monster()
	{
		std::vector<std::pair<int,int>> possibilities;

		for(int x = 0; x < MAP_WIDTH;x++)
		{
			int y=0;
			if(walkable(x,y) && dungeon_occupants[dungeon_index(x,y)] == NULL)
				possibilities.push_back(std::make_pair(x,y));
		}
		for(int x = 0; x < MAP_WIDTH;x++)
		{
			int y = MAP_HEIGHT-1;
			if(walkable(x,y) && dungeon_occupants[dungeon_index(x,y)] == NULL)
				possibilities.push_back(std::make_pair(x,y));
		}
		for(int y = 0; y < MAP_HEIGHT;y++)
		{
			int x = 0;
			if(walkable(x,y) && dungeon_occupants[dungeon_index(x,y)] == NULL)
				possibilities.push_back(std::make_pair(x,y));
		}
		for(int y = 0; y < MAP_HEIGHT;y++)
		{
			int x = MAP_WIDTH-1;
			if(walkable(x,y) && dungeon_occupants[dungeon_index(x,y)] == NULL)
				possibilities.push_back(std::make_pair(x,y));
		}
		if(possibilities.size()>0)
		{
			int r = rand()%15;
			while(r == 0 && bosses_generated >=boss_quota)
			{
				r = rand()%15;
			}
			int health;
			int weapon_id;
			char ch;
			int col;
			
			bool go_for_damage =false;
			bool go_for_restructure = false;
			if(game_mode != 't')
			{
				go_for_damage = (rand()%20==0);
				go_for_restructure = false;
			}
			int boss_count = num_bosses_existing();
			if(game_mode == 't')
			{
				r = 6;
				if(dungeon_level == 1)
				{
					if(enemies_killed[1] +num_bs_existing() >= 10)
						return;
				}
			}
			if(boss_count == 0 && (game_mode != 't' || dungeon_level != 1))
				r = 0;
			if(r<1)
			{
				if(boss_count > 0)
				{
					if(rand()%boss_count != 0)
						return;
				}
				weapon_id = patterns::monster_weapon_id(4);
				health = monster_health(4);//dungeon_level*dungeon_level;
				bosses_generated++;
				ch = 'E';
				col = COLOR_RED;
			}
			else if(r<3)
			{
				weapon_id = patterns::monster_weapon_id(3);
				health = monster_health(3);//dungeon_level*2;
				ch = 'd';
				col = COLOR_MAGENTA;
			}
			else if(r<6)
			{
				weapon_id = patterns::monster_weapon_id(2);
				health = monster_health(2);//dungeon_level*2;
				ch = 'c';
				col = COLOR_GREEN;
			}
			else if(r<10)
			{
				weapon_id = patterns::monster_weapon_id(1);
				health = monster_health(1);//dungeon_level*2;
				ch = 'b';
				col = COLOR_CYAN;
			}
			else
			{
				weapon_id = patterns::monster_weapon_id(0);
				health = monster_health(0);//dungeon_level;
				ch = 'a';
				col = COLOR_BLUE;
			}
			if(ch != 'E' && actors.size() >= (unsigned)(40+2*dungeon_level))
				return;
			std::pair<int,int> xy = possibilities[rand()%possibilities.size()];
			int restr_counter = 0;
			for(unsigned int i=0;i<actors.size();i++)
			{
				if(actors[i]->ch == ch)
				{
					if(actors[i]->go_for_restructuring) restr_counter++;
					if(restr_counter>=3) break;
				}
			}
			if(restr_counter<3 && (rand()%3==0) && game_mode != 't')
			{
				go_for_restructure = true;
			}
			actor* a = new actor(xy.first,xy.second,ch,col,health);
			if(go_for_damage && game_mode != 't') a->go_for_damage = true;
			else if(go_for_restructure&& game_mode != 't') a->go_for_restructuring = true;
			a->my_weapon = patterns::weapons[weapon_id];
			a->my_other_weapon = NULL;
			a->health = health;
			if(ch == 'a')
			{
				a->help_this_enemy_type = rand()%5;
			}
			dungeon_occupants[dungeon_index(xy.first,xy.second)] = a;
			actors.push_back(a);
		}
	}
	void do_damage(actor* a, actor* b, int damage)
	{
		flash_number(damage,a->x,a->y,COLOR_RED,b->x,b->y);
		if(a==p_ptr)
		{
			switch(b->ch)
			{
			case 'a': case 'A':
				damage_from_enemies[0] += damage;
				break;
			case 'b': case 'B':
				damage_from_enemies[1] += damage;
				break;
			case 'c': case 'C':
				damage_from_enemies[2] += damage;
				break;
			case 'd': case 'D':
				damage_from_enemies[3] += damage;
				break;
			case 'e': case 'E':
				damage_from_enemies[4] += damage;
				break;
			}
		}

		if(a==p_ptr)
			hurt_this_turn = true;
		a->health -= damage;
		if(a->health <= 0)
		{
			if(b == p_ptr)
			{
				switch(a->ch)
				{
					case 'a': case 'A':
						enemies_killed[0] ++;
						break;
					case 'b': case 'B':
						enemies_killed[1] ++;
						break;
					case 'c': case 'C':
						enemies_killed[2] ++;
						break;
					case 'd': case 'D':
						enemies_killed[3] ++;
						break;
					case 'e': case 'E':
						enemies_killed[4] ++;
						break;
				}
			}
			flash_number(0,a->x,a->y,COLOR_RED,b->x,b->y);
			if(a->ch == 'E')
			{
				draw_dungeon();
				draw_ui();
				mvaddstr(12,16,"                  ");
				mvaddstr(13,16," /--------------\\ ");
				mvprintw(14,16," | Boss killed! | ",dungeon_level);
				mvaddstr(15,16," \\--------------/ ");
				mvaddstr(16,16,"                  ");
				getch();
				bosses_killed++;
			}
			if(a->is_player)
			{
#ifdef DEBUG_NO_DAMAGE
				a->health = a->max_health;
#endif
				draw_dungeon();
				draw_ui();
			}
			dungeon::dungeon_occupants[dungeon::dungeon_index(a->x,a->y)] = NULL;
			
		}
		else
		{
			flash_number(a->health,a->x,a->y,COLOR_GREEN,b->x,b->y);
		}
		//debug
		/*if(p_ptr->health <= 0)
		{
			p_ptr->health = player_health_max(dungeon_level);
		}*/
	}
	void do_all_restructuring()
	{
		for(int x=0;x<MAP_WIDTH;x++)
		{
			for(int y=0;y<MAP_HEIGHT;y++)
			{
				int d = (int)dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t;
				if(d <= 9 && d >= 1)
				{
					actor* a = dungeon_occupants[dungeon_index(x,y)];
					if(a != NULL)
					{
						if(rand()%3 == 0)
							patterns::dungeon_restructure_real(a,d);
						if(a->go_for_restructuring)
						{
							switch(a->ch)
							{
							case 'a':
								a->ch = 'A'; break;
							case 'b':
								a->ch = 'B'; break;
							case 'c':
								a->ch = 'C'; break;
							case 'd':
								a->ch = 'D'; break;
							case 'E':
								a->ch = 'E'; break;
							}
						}
					}
				}
			}
		}
	}
}


bool los_exists(int x1, int y1, int x2, int y2)
{
    // if x1 == x2 or y1 == y2, then it does not matter what we set here
    int delta_x(x2 - x1);
    signed char ix((delta_x > 0) - (delta_x < 0));
    delta_x = std::abs(delta_x) << 1;
 
    int delta_y(y2 - y1);
    signed char iy((delta_y > 0) - (delta_y < 0));
    delta_y = std::abs(delta_y) << 1;
 
    if (delta_x >= delta_y)
    {
        // error may go below zero
        int error(delta_y - (delta_x >> 1));
 
        while (x1 != x2)
        {
            if (error >= 0)
            {
                if (error || (ix > 0))
                {
                    y1 += iy;
                    error -= delta_x;
                }
                // else do nothing
            }
            // else do nothing
 
            x1 += ix;
            error += delta_y;
			if ((x1 != x2) || (y1 != y2))//(x1 != x2 && y1 != y2)
			{
				if (dungeon::dungeon_terrain[dungeon::dungeon_index(x1, y1)].t ==terrains::WALL) return false;
			}
        }
    }
    else
    {
        // error may go below zero
        int error(delta_x - (delta_y >> 1));
 
        while (y1 != y2)
        {
            if (error >= 0)
            {
                if (error || (iy > 0))
                {
                    x1 += ix;
                    error -= delta_y;
                }
                // else do nothing
            }
            // else do nothing
 
            y1 += iy;
            error += delta_x;
 
            if ((x1 != x2) || (y1 != y2))//(x1 != x2 && y1 != y2)
			{
				if (dungeon::dungeon_terrain[dungeon::dungeon_index(x1, y1)].t ==terrains::WALL) return false;
			}
        }
    }
	return true;
}

bool symmetrical_los(int x1, int y1, int x2, int y2)
{
	return (abs(std::max(x1-x2,y1-y2)) <= 8) && (los_exists(x1, y1, x2, y2) || los_exists(x2, y2, x1, y1));
}

namespace movement
{
	std::pair<int,int> delta(int dir)
	{
		std::pair<int,int> r = std::make_pair(0,0);
		switch(dir)
		{
		case 0: case 1: case 2:
			r.second = 1;
			break;
		case 4:case 5:case 6:
			r.second = -1;
		}
		switch(dir)
		{
		case 2:case 3:case 4:
			r.first = 1;
			break;
		case 6: case 7: case 0:
			r.first = -1;
		}
		return r;
	}
	
	bool move_actor(int dir, actor* a)
	{
		if (dir<-1 || dir > 7)
			return false;
		if(dir == -1)
		{
			if(rand()%100 < std::min(50,3*(dungeon_level_number(dungeon_level))))
			{
				if(a->health < a-> max_health)
				{
					a->health++;
					if(a == p_ptr)
						regen_this_turn = true;
				}
			}
			return true;
		}
		std::pair<int,int> del = delta(dir);
		int x = a->x+del.first;
		int y = a->y+del.second;
		if (!dungeon::walkable(x,y))
			return false;
		if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] != NULL)
			return false;
		dungeon::dungeon_occupants[dungeon::dungeon_index(a->x,a->y)] = NULL;
		a->x = x;
		a->y = y;
		dungeon::dungeon_occupants[dungeon::dungeon_index(a->x,a->y)] = a;
		return true;
	}

	void bfs(int sx=-1,int sy=-1)
	{
		if(sx==-1) sx = p_ptr->x;
		if(sy==-1) sy = p_ptr->y;
		for(int x=0;x<dungeon::MAP_WIDTH;x++)
		{
			for(int y=0;y<dungeon::MAP_HEIGHT;y++)
			{
				dungeon::distance_to_player[dungeon::dungeon_index(x,y)] = dungeon::MAP_WIDTH+dungeon::MAP_HEIGHT+1;
			}
		}
		std::vector<std::pair<int,int>> border;
		border.push_back(std::make_pair(sx,sy));
		dungeon::distance_to_player[dungeon::dungeon_index(sx,sy)] = 0;
		std::vector<std::pair<int,int>> next_border;
		while(border.size()>0)
		{
			for(unsigned int i=0;i<border.size();i++)
			{
				int x = border[i].first;
				int y = border[i].second;
				int dist = dungeon::distance_to_player[dungeon::dungeon_index(x,y)];
				for(int dir=0;dir<8;dir++)
				{
					int xx = x+delta(dir).first;
					int yy = y+delta(dir).second;
					if(dungeon::walkable(xx,yy))
					{
						if(dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] > dist+1)
						{
							dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] = dist+1;
							if(dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)] != NULL)
							{
								if(!dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)]->dont_want_to_move)
								{
									next_border.push_back(std::make_pair(xx,yy));
								}
							}
							else
							{
								next_border.push_back(std::make_pair(xx,yy));
							}
						}
					}
				}
			}
			border.clear();
			border.insert(border.begin(),next_border.begin(),next_border.end());
			next_border.clear();
		}
	}

	void random_walk_ai(actor* a)
	{
		if(rand()%9 == 0)
		{
			move_actor(-1,a);
			return;
		}
		std::vector<int> can_go;
		for(int i=0;i<8;i++)
		{
			int xx = a->x + delta(i).first;
			int yy = a->y + delta(i).second;
			if(dungeon::walkable(xx,yy))
				if(dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)] == NULL)
					can_go.push_back(i);
		}
		if(can_go.size()>0)
		{
			int dir = can_go[rand()%can_go.size()];
			move_actor(dir,a);
		}
		else
		{
			move_actor(-1,a);
		}
	}
	void restructure_ai(actor* a)
	{
		int t = (int)dungeon::dungeon_terrain[dungeon::dungeon_index(a->x,a->y)].t;
		if(t < 1 || t > 9)
		{
			random_walk_ai(a); a->dont_want_to_move = false; return;
		}
		else
		{
			a->dont_want_to_move = true;
			move_actor(-1,a);
		}
	}
	void surround_player_ai(actor* a, bool attempt_to_collide=false)
	{
		if(!attempt_to_collide && abs(a->x - p_ptr->x) < 2 && abs(a->y - p_ptr->y) < 2)
		{
			a->dont_want_to_move = (rand()%2==0);
			move_actor(-1,a);
		}
		else
		{
			a->dont_want_to_move = false;
			std::vector<int> best_list;
			int best_dist = dungeon::MAP_WIDTH*dungeon::MAP_HEIGHT+1;
			for(int i=0;i<8;i++)
			{
				int xx = a->x+delta(i).first;
				int yy = a->y+delta(i).second;
				if(dungeon::walkable(xx,yy))
				{
					if(dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)] == NULL)
					{
						if(dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] < best_dist)
						{
							best_list.clear();
							best_list.push_back(i);
							best_dist = dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)];
						}
						else if(dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] == best_dist)
						{
							best_list.push_back(i);
						}
					}
				}
			}
			if(best_list.size() != 0)
			{
				move_actor(best_list[rand()%best_list.size()],a);
			}
			else
			{
				random_walk_ai(a);
			}
		}
	}
	void block_safe_ai(actor* a)
	{
		if((abs(p_ptr->x - a->x) + (p_ptr->y - a->y)) > 8)
		{
			surround_player_ai(a,true);  a->dont_want_to_move = false; return;
		}
		else if(!dungeon::enemies_can_attack[a->help_this_enemy_type][dungeon::dungeon_index(a->x,a->y)])
		{
			random_walk_ai(a); a->dont_want_to_move = false; return;
		}
		else
		{
			a->dont_want_to_move = true;
			move_actor(-1,a);
		}
	}

	void prevent_attack_ai(actor* a)
	{
		if((abs(p_ptr->x - a->x) + (p_ptr->y - a->y)) > 8)
		{
			surround_player_ai(a,true);  a->dont_want_to_move = false; return;
		}
		else if(!(dungeon::player_can_attack_from[dungeon::dungeon_index(a->x,a->y)] || dungeon::player_can_attack_from_other[dungeon::dungeon_index(a->x,a->y)]))
		{
			random_walk_ai(a); a->dont_want_to_move = false; return;
		}
		else
		{
			a->dont_want_to_move = true;
			move_actor(-1,a);
		}
	}
	void camp_damage_ai(actor* a)
	{
		int e = -1;
		switch(a->ch)
		{
		case 'a': case 'A':
			e=0;
			break;
		case 'b': case 'B':
			e=1;
			break;
		case 'c': case 'C':
			e=2;
			break;
		case 'd': case 'D':
			e=3;
			break;
		case 'e': case 'E':
			e=4;
			break;
		}
		if((abs(p_ptr->x - a->x) + (p_ptr->y - a->y)) > 8)
		{
			surround_player_ai(a,true);  a->dont_want_to_move = false; return;
		}
		else if(!(dungeon::enemies_can_attack_from[e][dungeon::dungeon_index(a->x,a->y)]))
		{
			random_walk_ai(a); a->dont_want_to_move = false; return;
		}
		else
		{
			a->dont_want_to_move = true;
			move_actor(-1,a);
		}
	}
	void damage_ai(actor* a)
	{
		camp_damage_ai(a);
		return;
		/*int best_x_player=-1;
		int best_y_player=-1;
		int best_distance_player=dungeon::MAP_HEIGHT+dungeon::MAP_WIDTH+1;
		int best_x_actor=-1;
		int best_y_actor=-1;
		int best_distance_actor=dungeon::MAP_HEIGHT+dungeon::MAP_WIDTH+1;
		for(int x=0;x<dungeon::MAP_WIDTH;x++)
		{
			for(int y=0;y<dungeon::MAP_HEIGHT;y++)
			{
				if(dungeon::enemies_can_attack_from[dungeon::dungeon_index(x,y)])
				{
					int distance_player = std::max(abs(x-p_ptr->x),abs(y-p_ptr->y));
					int distance_actor = std::max(abs(x-a->x),abs(y-a->y));
					if(distance_actor < best_distance_actor)
					{
						best_distance_actor = distance_actor;
						best_x_actor = x;
						best_y_actor = y;
					}
					if(distance_player < best_distance_player)
					{
						best_distance_player = distance_player;
						best_x_player = x;
						best_y_player = y;
					}
				}
				if(dungeon::distance_to_player[dungeon::dungeon_index(a->x,a->y)] > 6)
				{
					bfs(best_x_player,best_y_player);
					surround_player_ai(a,true);
					bfs();
				}

			}
		}*/

	}
	void run_ai(actor* a)
	{
		if((abs(p_ptr->x - a->x) + (p_ptr->y - a->y)) > 10)
		{
			if(a->health < a->max_health && (rand()%3) == 0)
				move_actor(-1,a);
			else
				random_walk_ai(a);
			return;
		}
		a->dont_want_to_move = false;
		//int best_dir = -1;
		int best_dist = 0;
		std::vector<int> best_list;
		for(int i=0;i<8;i++)
		{
			int xx = a->x+delta(i).first;
			int yy = a->y+delta(i).second;
			if(dungeon::walkable(xx,yy))
			{
				if(dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)] == NULL)
				{
					if(dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] > best_dist)
					{
						best_list.clear();
						best_list.push_back(i);
						best_dist = dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)];
					}
					else if(dungeon::distance_to_player[dungeon::dungeon_index(xx,yy)] == best_dist)
					{
						best_list.push_back(i);
					}
				}
			}
		}
		if(best_list.size() != 0)
		{
			move_actor(best_list[rand()%best_list.size()],a);
		}
		else
		{
			random_walk_ai(a);
		}
	}
}

namespace patterns
{
	struct pattern {terrains::terrain_class p[8]; bool rotations; bool reflections;};
	struct weapon { bool must_be_visible; bool must_be_adjacent; bool all_matching; int damage; int pu; int pv; pattern user_pattern; pattern victim_pattern;  long long int quality;int id;};
	bool lower_quality(weapon* w1, weapon* w2)
	{
		return w1->quality < w2->quality;
	}
	bool match_pattern(pattern p, int x, int y)
	{
		for(int d=0;d<8;d++)
		{
			std::pair<int,int> del = movement::delta(d);
			int xx = x+del.first;
			int yy = y+del.second;
			if(dungeon::on_map(xx,yy))
			{
				if(!terrains::terrain_matches_class(p.p[d], dungeon::dungeon_terrain[dungeon::dungeon_index(xx,yy)]))
					return false;
			}
		}
		return true;
	}
	pattern rotate_pattern(pattern p, int r)
	{
		pattern q = p;
		for(int i=0;i<8;i++)
		{
			q.p[i] = p.p[(i+r)%8];
		}
		return q;
	}
	pattern reflect_pattern(pattern p)
	{
		pattern q = p;
		for(int i=0;i<8;i++)
		{
			q.p[7-i] = p.p[i];
		}
		return q;
	}
	bool match_pattern_alternatives(pattern p, int x, int y)
	{
		if(!p.rotations)
		{
			return match_pattern(p,x,y);
		}
		else
		{
			for(int r=0;r<8;r++)
			{
				pattern q = rotate_pattern(p,r);
				if(match_pattern(q,x,y)) return true;
				if(p.reflections)
					if(match_pattern(reflect_pattern(q),x,y)) return true;
			}
		}
		return false;
	}
	bool actor_targettable(weapon* w,actor* u, actor* v)
	{
		if(w->must_be_adjacent)
		{
			if(abs(u->x - v->x)>1) return false;
			if(abs(u->y-v->y)>1) return false;
			return true;
		}
		else if(w->must_be_visible)
		{
			return symmetrical_los(u->x,u->y,v->x,v->y);
		}
		return (abs(std::max(u->x-v->x,u->y-v->y)) <= 8);
	}
	void use_weapon(weapon* w, actor* a)
	{
		if(match_pattern_alternatives(w->user_pattern,a->x,a->y))
		{
			if(a->is_player)
			{
				std::vector<actor*> targets;
				for(unsigned int i=0;i<actors.size();i++)
				{
					if(actors[i]->health<=0) continue;
					if(!actors[i]->is_player)
					{
						if(actor_targettable(w,a,actors[i]))
						{
							if(match_pattern_alternatives(w->victim_pattern,actors[i]->x,actors[i]->y))
							{
								if(w->all_matching)
									dungeon::do_damage(actors[i],a,w->damage);
								else
									targets.push_back(actors[i]);
							}
						}
					}
				}
				if(!w->all_matching &&targets.size()>0)
				{
					int r = rand()%targets.size();
					dungeon::do_damage(targets[r],a,w->damage);
				}
			}
			else
			{
				if(actor_targettable(w,a,p_ptr))
				{
					if(match_pattern_alternatives(w->victim_pattern,p_ptr->x,p_ptr->y))
					{
						if(a->go_for_damage)
						{
							if(a->ch == 'a')
								a->ch = 'A';
							else if(a->ch == 'b')
								a->ch = 'B';
							else if(a->ch == 'c')
								a->ch = 'C';
							else if(a->ch == 'd')
								a->ch = 'D';
						}
						dungeon::do_damage(p_ptr,a,w->damage);
					}
				}
			}
		}
	}
	long long int calculate_weapon_quality(weapon* w)
	{
		int pu = 0; // probability of valid location for user times 256
		int pv = 0; // probability of valid location for victim times 256
		pattern weapon_patterns_user[16];
		pattern weapon_patterns_victim[16];
		for(int i=0;i<16;i++)
		{
			weapon_patterns_user[i] = i < 8 ? rotate_pattern(w->user_pattern,i) : reflect_pattern(rotate_pattern(w->user_pattern,i-8));
			weapon_patterns_victim[i] = i < 8 ? rotate_pattern(w->victim_pattern,i) : reflect_pattern(rotate_pattern(w->victim_pattern,i-8));
		}
		int user_patterns_to_try = w->user_pattern.rotations? (w->user_pattern.reflections?16:8) : 1;
		int victim_patterns_to_try = w->victim_pattern.rotations? (w->victim_pattern.reflections?16:8) : 1;
		for(int i=0;i<256;i++)
		{
			bool found_match = false;
			for(int j=0; j<user_patterns_to_try && !found_match;j++)
			{
				bool match_possible = true;
				for(int k=0;k<8 && match_possible;k++)
				{
					bool wall = ((i & (1 << k)) != 0);
					terrains::terrain_class t = weapon_patterns_user[j].p[k];
					if(t == terrains::CFLOOR && wall) match_possible = false;
					if(t == terrains::CWALL && !wall) match_possible = false;

				}
				if(match_possible)
				{
					found_match = true;
					pu++;
				}
			}
			found_match = false;
			for(int j=0; j<victim_patterns_to_try && !found_match;j++)
			{
				bool match_possible = true;
				for(int k=0;k<8 && match_possible;k++)
				{
					bool wall = ((i & (1 << k)) != 0);
					terrains::terrain_class t = weapon_patterns_victim[j].p[k];
					if(t == terrains::CFLOOR && wall) match_possible = false;
					if(t == terrains::CWALL && !wall) match_possible = false;

				}
				if(match_possible)
				{
					found_match = true;
					pv++;
				}
			}
		}
		long long int pupvd = (long)pu*(long)pv*(long)w->damage;
		w->pu = pu;
		w->pv = pv;

		if(w->must_be_adjacent)
		{
			if(w->all_matching) // up to 8, difficult to get to
			{
				return pupvd*16L;
			}
			else
			{
				return pupvd/2;
			}
		}
		else
		{
			if(w->must_be_visible)
			{
				if(w->all_matching) // several
				{
					return pupvd*128L;
				}
				else
				{
					return pupvd*12L;
				}
			}
			else
			{
				if(w->all_matching) // everyone
				{
					return pupvd*128L*64L;
				}
				else
				{
					return pupvd*128L;
				}
			}
		}
	}
		int decide_determined_tiles(bool user)
	{
		if(!user)
		{
			int r = rand()%1350;
			if(r<10)
				return 8;
			else if(r<40)
				return 7;
			else if(r<100)
				return 6;
			else if(r<200)
				return 5;
			else if(r<350)
				return 4;
			else if(r<600)
				return 3;
			else if(r<1000)
				return 2;
			else if(r<1200)
				return 1;
			else if(r<1350)
				return 0;
			return 0;
		}
		else
		{
			int r = rand()%2000;
			if(r<30)
				return 8;
			else if(r<90)
				return 7;
			else if(r<210)
				return 6;
			else if(r<510)
				return 5;
			else if(r<1110)
				return 4;
			else if(r<1510)
				return 3;
			else if(r<1760)
				return 2;
			else if(r<1910)
				return 1;
			else if(r<2000)
				return 0;
			return 0;
		}
	}
	pattern generate_pattern(bool user)
	{
		int count = decide_determined_tiles(user);
		pattern p;
		for(int i=0;i<8;i++)
		{
			if(i<count)
			{
				p.p[i] = (rand()%2==0)?terrains::CFLOOR : terrains::CWALL;
			}
			else
			{
				p.p[i] = terrains::CANY;
			}
		}
		for(int i=0;i<8;i++)
		{
			int r = (rand()%(8-i))+i;
			terrains::terrain_class swap = p.p[r];
			p.p[r] = p.p[i];
			p.p[i] = swap;
		}
		int r = rand()%100;
		p.rotations = false;
		p.reflections = false;
		if(user)
		{
			if(r<40)
			{ }
			else
			{
				p.rotations = true;
				if(r<80)
				{
					p.reflections = true;
				}
			}
		}
		else
		{
			if(r<25)
			{ }
			else
			{
				p.rotations = true;
				if(r<75)
				{
					p.reflections = true;
				}
			}
		}
		return p;
	}
	
	weapon* generate_weapon()
	{
		weapon* w = new weapon;
		int cu = decide_determined_tiles(true);
		int cv = decide_determined_tiles(false);
		w->user_pattern = generate_pattern(true);
		w->victim_pattern = generate_pattern(false);
		w->all_matching = (rand()%5 == 0);
		int r = rand()%100;
		if(r<35)
		{
			w->must_be_adjacent = true;
			w->must_be_visible = true;
		}
		else if(r<85)
		{
			w->must_be_adjacent = false;
			w->must_be_visible = true;
		}
		else
		{
			w->must_be_adjacent = false;
			w->must_be_visible = false;
		}
		w->damage = rand()%10 + 1;
		switch(w->damage)
		{
		case 1:
			w->damage = 1; break;
		case 2:
			w->damage = 1; break;
		case 3:
			w->damage = 2; break;
		case 4:
			w->damage = 3; break;
		case 5:
			w->damage = 5; break;
		case 6:
			w->damage = 8; break;
		case 7:
			w->damage = 13; break;
		case 8:
			w->damage = 21; break;
		case 9:
			w->damage = 34; break;
		case 10:
			w->damage = 55; break;
		}
		w->quality = calculate_weapon_quality(w);
		return w;
	}
	void dungeon_restructure_step(weapon* w)
	{
		int x = rand()%(dungeon::MAP_WIDTH-2)+1;
		int y = rand()%(dungeon::MAP_HEIGHT-2)+1;
		pattern p = (rand()%2==0)?w->user_pattern:w->victim_pattern;
		std::vector<pattern> options;
		
		if(p.rotations)
		{
			for(int r=0;r<8;r++)
			{
				options.push_back(rotate_pattern(p,r));
				if(p.reflections)
					options.push_back(reflect_pattern(rotate_pattern(p,r)));
			}
		}
		else
		{
			options.push_back(p);
		}
		if(!dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].permanent)
			if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] == NULL)
				if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] == NULL)
					dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t =terrains::FLOOR;
		pattern q = options[rand()%options.size()];
		for(int i=0;i<8;i++)
		{
			std::pair<int,int> del = movement::delta(i);
			int xx = x+del.first;
			int yy = y+del.second;
			if(q.p[i]==terrains::CANY) continue;
			if(!dungeon::dungeon_terrain[dungeon::dungeon_index(xx,yy)].permanent)
				if(dungeon::dungeon_occupants[dungeon::dungeon_index(xx,yy)] == NULL)
					if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] == NULL)
						dungeon::dungeon_terrain[dungeon::dungeon_index(xx,yy)].t = (q.p[i]==terrains::CWALL?terrains::WALL : terrains::FLOOR);
		}

	}
	weapon* tutorial_weapon(int id)
	{
		weapon* w = new weapon;
		pattern up;
		pattern tp;
		w->all_matching = false;
		switch(id)
		{
		case 0:
			for(int i=0;i<8;i++)
			{
				up.p[i] = (i%2==0)? terrains::CFLOOR : terrains::CWALL;
				
				tp.p[i] = ((i==3 || i==7)?terrains::CWALL : terrains::CFLOOR);
			}
			w->damage = 1;
			w->must_be_adjacent = true;
			w->must_be_visible = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = false;
			w->id = 0;
			break;
		case 1:
			for(int i=0;i<8;i++)
			{
				tp.p[i] = (i%2!=0)?terrains::CFLOOR : terrains::CWALL;
				up.p[i] = ((i==1 || i==5)?terrains::CWALL : terrains::CFLOOR);
			}
			up.p[3] = terrains::CANY;
			up.p[7] = terrains::CANY;
			w->damage = 1;
			w->must_be_adjacent = true;
			w->must_be_visible = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = false;
			w->id = 1;
			break;
		case 2:
			for(int i=0;i<8;i++)
			{
				tp.p[i] = terrains::CANY;
				up.p[i] = terrains::CANY;
			}
			up.p[3] = terrains::CFLOOR;
			up.p[4] = terrains::CWALL;
			up.p[5] = terrains::CFLOOR;
			tp.p[0] = terrains::CWALL;
			tp.p[3] = terrains::CWALL;
			w->damage = 21;
			w->must_be_adjacent = true;
			w->must_be_visible = true;
			w->all_matching = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = true;
			w->id = 2;
			break;
		case 3:
			for(int i=0;i<8;i++)
			{
				tp.p[i] = terrains::CANY;
				up.p[i] = terrains::CANY;
			}
			up.p[3] = terrains::CFLOOR;
			up.p[4] = terrains::CWALL;
			up.p[5] = terrains::CWALL;
			up.p[6] = terrains::CWALL;
			up.p[7] = terrains::CFLOOR;
			tp.p[0] = terrains::CFLOOR;
			tp.p[1] = terrains::CWALL;
			tp.p[2] = terrains::CFLOOR;
			w->damage = 5;
			w->must_be_adjacent = false;
			w->must_be_visible = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = true;
			w->id = 3;
			break;
		case 4:
			for(int i=0;i<8;i++)
			{
				tp.p[i] = terrains::CANY;
				up.p[i] = terrains::CANY;
			}
			up.p[3] = terrains::CWALL;
			tp.p[0] = terrains::CWALL;
			w->damage = 1;
			w->must_be_adjacent = true;
			w->must_be_visible = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = false;
			w->id = 4;
			break;
		case 5:
			for(int i=0;i<8;i++)
			{
				tp.p[i] = terrains::CANY;
				up.p[i] = terrains::CANY;
			}
			up.p[5] = terrains::CWALL;
			up.p[6] = terrains::CFLOOR;
			tp.p[7] = terrains::CWALL;
			tp.p[0] = terrains::CFLOOR;
			tp.p[1] = terrains::CWALL;
			w->damage = 1;
			w->must_be_adjacent = true;
			w->must_be_visible = true;
			up.rotations = true;
			tp.rotations = true;
			up.reflections = false;
			tp.reflections = true;
			w->id = 5;
			break;
		}
		
		w->user_pattern = up;
		w->victim_pattern = tp;
		w->quality = calculate_weapon_quality(w);
		return w;
	}
	void dungeon_restructure_real(actor* a, int count)
	{
		for (int i=0;i<count;i++)
		{
			if(a->my_other_weapon != NULL)
			{
				dungeon_restructure_step(rand()%2==0?a->my_weapon:a->my_other_weapon);
			}
			else
			{
				dungeon_restructure_step(a->my_weapon);
			}
		}
	}

	void note_all(bool player_only,int enemies)
	{
		for(int x=0;x<dungeon::MAP_WIDTH;x++)
		{
			for(int y=0;y<dungeon::MAP_HEIGHT;y++)
			{
				if(patterns::match_pattern_alternatives(p_ptr->my_weapon->victim_pattern,x,y))
				{
					dungeon::player_can_attack[dungeon::dungeon_index(x,y)] = true;
				}
				else
				{
					dungeon::player_can_attack[dungeon::dungeon_index(x,y)] = false;
				}
				if(patterns::match_pattern_alternatives(p_ptr->my_weapon->user_pattern,x,y))
				{
					dungeon::player_can_attack_from[dungeon::dungeon_index(x,y)] = true;
				}
				else
				{
					dungeon::player_can_attack_from[dungeon::dungeon_index(x,y)] = false;
				}
				if(true)//if(game_mode != 't') // if(p_ptr->my_other_weapon != NULL)
				{
					if(patterns::match_pattern_alternatives(p_ptr->my_other_weapon->victim_pattern,x,y))
					{
						dungeon::player_can_attack_other[dungeon::dungeon_index(x,y)] = true;
					}
					else
					{
						dungeon::player_can_attack_other[dungeon::dungeon_index(x,y)] = false;
					}

					if(patterns::match_pattern_alternatives(p_ptr->my_other_weapon->user_pattern,x,y))
					{
						dungeon::player_can_attack_from_other[dungeon::dungeon_index(x,y)] = true;
					}
					else
					{
						dungeon::player_can_attack_from_other[dungeon::dungeon_index(x,y)] = false;
					}
				}
				if(!player_only)
				{
					int a,b;
					if(enemies==-1)
					{
						a=0;b=5;
					}
					else
					{
						a=enemies; b = enemies+1;
					}
					if(game_mode != 't')
					{
						for(int e=a;e<b;e++)
						{
							if(patterns::match_pattern_alternatives(patterns::weapons[patterns::monster_weapon_id(e)]->victim_pattern,x,y))
							{
								dungeon::enemies_can_attack[e][dungeon::dungeon_index(x,y)] = true;
							}
							else
							{
								dungeon::enemies_can_attack[e][dungeon::dungeon_index(x,y)] = false;
							}
							if(patterns::match_pattern_alternatives(patterns::weapons[patterns::monster_weapon_id(e)]->user_pattern,x,y))
							{
								dungeon::enemies_can_attack_from[e][dungeon::dungeon_index(x,y)] = true;
							}
							else
							{
								dungeon::enemies_can_attack_from[e][dungeon::dungeon_index(x,y)] = false;
							}
						}
					}
				}
			}
		}
	}
}

//void draw_dungeon_enemies()
//{
//	for(int y=0; y<dungeon::MAP_HEIGHT;y++)
//	{
//		for(int x=0; x<dungeon::MAP_WIDTH;x++)
//		{
//			if(dungeon::walkable(x,y))
//			{
//				bool enemy_can_attack_from_here = false;
//				bool enemy_could_attack_here = false;
//				bool enemy_exists[5];
//				for(int e=0;e<5;e++) enemy_exists[e] = false;
//				for(unsigned int i=0; i< actors.size();i++)
//				{
//					if(!actors[i]->is_player)
//					{
//						switch(actors[i]->ch)
//						{
//						case 'a': case 'A':
//							enemy_exists[0] = true;
//						case 'b': case 'B':
//							enemy_exists[1] = true;
//						case 'c': case 'C':
//							enemy_exists[2] = true;
//						case 'd': case 'D':
//							enemy_exists[3] = true;
//						case 'e': case 'E':
//							enemy_exists[4] = true;
//						}
//					}
//				}
//				for(int e=0;e<5;e++)
//				{
//					if(!enemy_exists[e]) continue;
//				
//					if(dungeon::enemies_can_attack_from[e][dungeon::dungeon_index(x,y)])
//					{
//						enemy_can_attack_from_here = true;
//					}
//					if(dungeon::enemies_can_attack[e][dungeon::dungeon_index(x,y)])
//					{
//						enemy_could_attack_here = true;
//					}
//				}
//				if(enemy_could_attack_here)
//				{
//					if(enemy_can_attack_from_here)
//					{
//						attron(COLOR_PAIR(COLOR_CYAN));
//					}
//					else
//					{
//						attron(COLOR_PAIR(COLOR_MAGENTA));
//					}
//				}
//				else
//				{
//					if(enemy_can_attack_from_here)
//					{
//						attron(COLOR_PAIR(COLOR_BLUE));
//					}
//					else
//					{
//						attron(COLOR_PAIR(COLOR_WHITE));
//					}
//				}
//			}
//			if(symmetrical_los(p_ptr->x,p_ptr->y,x,y))
//			{
//				attron(A_BOLD);
//			}
//			if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] != NULL)
//			{
//				mvaddch(y,x,dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->ch);
//			}
//			else if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
//			{
//				mvaddch(y,x,'^');
//			}
//			else
//			{
//				mvaddch(y,x,terrains::terrain_char( dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t));
//			}
//			attron(COLOR_PAIR(COLOR_WHITE));
//			attroff(A_BOLD);
//		}
//	}
//	refresh();
//}

void draw_dungeon_player()
{
	bool player_can_attack = dungeon::player_can_attack_from[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];// patterns::match_pattern_alternatives(p_ptr->my_weapon->user_pattern,p_ptr->x,p_ptr->y);
	//bool player_can_attack_other = dungeon::player_can_attack_from_other[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];//patterns::match_pattern_alternatives(p_ptr->my_other_weapon->user_pattern,p_ptr->x,p_ptr->y);
	for(int y=0; y<dungeon::MAP_HEIGHT;y++)
	{
		for(int x=0; x<dungeon::MAP_WIDTH;x++)
		{
			bool in_range = true;
			//bool in_range_other = true;
			if(p_ptr->my_weapon->must_be_adjacent)
				in_range = (abs(p_ptr->x-x)<=1) & (abs(p_ptr->y-y)<= 1);
			else if(p_ptr->my_weapon->must_be_visible)
				in_range = symmetrical_los(p_ptr->x,p_ptr->y,x,y);

			/*if(p_ptr->my_other_weapon->must_be_adjacent)
				in_range_other = (abs(p_ptr->x-x)<=1) & (abs(p_ptr->y-y)<= 1);
			else if(p_ptr->my_other_weapon->must_be_visible)
				in_range_other = symmetrical_los(p_ptr->x,p_ptr->y,x,y);*/

			bool attackable = dungeon::player_can_attack[dungeon::dungeon_index(x,y)];//patterns::match_pattern_alternatives(p_ptr->my_weapon->victim_pattern,x,y);
			//bool attackable_other =  dungeon::player_can_attack_other[dungeon::dungeon_index(x,y)];// patterns::match_pattern_alternatives(p_ptr->my_other_weapon->victim_pattern,x,y);
			bool good_base = dungeon::player_can_attack_from[dungeon::dungeon_index(x,y)];// patterns::match_pattern_alternatives(p_ptr->my_weapon->user_pattern,x,y);
			//bool good_base_other = dungeon::player_can_attack_from_other[dungeon::dungeon_index(x,y)]; //patterns::match_pattern_alternatives(p_ptr->my_other_weapon->user_pattern,x,y);
			bool really_attackable = attackable && in_range && player_can_attack;
			//bool really_attackable_other = attackable_other && in_range_other && player_can_attack_other;

			if(symmetrical_los(p_ptr->x,p_ptr->y,x,y))
			{
				attron(A_BOLD);
			}
			if(really_attackable)
			{
				if(good_base)
				{
					attron(COLOR_PAIR(COLOR_GREEN));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_RED));
				}
			}
			else if(attackable)
			{
				if(good_base)
				{
					attron(COLOR_PAIR(COLOR_CYAN));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_MAGENTA));
				}
			}
			else
			{
				if(good_base)
				{
					attron(COLOR_PAIR(COLOR_BLUE));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_WHITE));
				}
			}

			if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] != NULL)
			{
				mvaddch(y,x,dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->ch);
			}
			else if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
			{
				mvaddch(y,x,'^');
			}
			else
			{
				if(!dungeon::walkable(x,y))
					attron(COLOR_PAIR(COLOR_WHITE));
				mvaddch(y,x,terrains::terrain_char( dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t));
			}
			attroff(A_BOLD);
			attron(COLOR_PAIR(COLOR_WHITE));
		}
	}
	refresh();
}

void draw_dungeon_player_other()
{
	//bool player_can_attack = dungeon::player_can_attack_from[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];// patterns::match_pattern_alternatives(p_ptr->my_weapon->user_pattern,p_ptr->x,p_ptr->y);
	bool player_can_attack_other = dungeon::player_can_attack_from_other[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];//patterns::match_pattern_alternatives(p_ptr->my_other_weapon->user_pattern,p_ptr->x,p_ptr->y);
	for(int y=0; y<dungeon::MAP_HEIGHT;y++)
	{
		for(int x=0; x<dungeon::MAP_WIDTH;x++)
		{
			//bool in_range = true;
			bool in_range_other = true;
			/*if(p_ptr->my_weapon->must_be_adjacent)
				in_range = (abs(p_ptr->x-x)<=1) & (abs(p_ptr->y-y)<= 1);
			else if(p_ptr->my_weapon->must_be_visible)
				in_range = symmetrical_los(p_ptr->x,p_ptr->y,x,y);*/

			if(p_ptr->my_other_weapon->must_be_adjacent)
				in_range_other = (abs(p_ptr->x-x)<=1) & (abs(p_ptr->y-y)<= 1);
			else if(p_ptr->my_other_weapon->must_be_visible)
				in_range_other = symmetrical_los(p_ptr->x,p_ptr->y,x,y);

			//bool attackable = dungeon::player_can_attack[dungeon::dungeon_index(x,y)];//patterns::match_pattern_alternatives(p_ptr->my_weapon->victim_pattern,x,y);
			bool attackable_other =  dungeon::player_can_attack_other[dungeon::dungeon_index(x,y)];// patterns::match_pattern_alternatives(p_ptr->my_other_weapon->victim_pattern,x,y);
			//bool good_base = dungeon::player_can_attack_from[dungeon::dungeon_index(x,y)];// patterns::match_pattern_alternatives(p_ptr->my_weapon->user_pattern,x,y);
			bool good_base_other = dungeon::player_can_attack_from_other[dungeon::dungeon_index(x,y)]; //patterns::match_pattern_alternatives(p_ptr->my_other_weapon->user_pattern,x,y);
			//bool really_attackable = attackable && in_range && player_can_attack;
			bool really_attackable_other = attackable_other && in_range_other && player_can_attack_other;

			if(symmetrical_los(p_ptr->x,p_ptr->y,x,y))
			{
				attron(A_BOLD);
			}
			if(really_attackable_other)
			{
				if( good_base_other)
				{
					attron(COLOR_PAIR(COLOR_GREEN));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_RED));
				}
			}
			else if(attackable_other)
			{
				if(good_base_other)
				{
					attron(COLOR_PAIR(COLOR_CYAN));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_MAGENTA));
				}
			}
			else
			{
				if(good_base_other)
				{
					attron(COLOR_PAIR(COLOR_BLUE));
				}
				else
				{
					attron(COLOR_PAIR(COLOR_WHITE));
				}
			}

			if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] != NULL)
			{
				mvaddch(y,x,dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->ch);
			}
			else if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
			{
				mvaddch(y,x,'^');
			}
			else
			{
				if(!dungeon::walkable(x,y))
					attron(COLOR_PAIR(COLOR_WHITE));
				mvaddch(y,x,terrains::terrain_char( dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t));
			}
			attroff(A_BOLD);
			attron(COLOR_PAIR(COLOR_WHITE));
		}
	}
	refresh();
}

void draw_tile_normal(int x, int y)
{
	if(symmetrical_los(p_ptr->x,p_ptr->y,x,y))
		{
			attron(A_BOLD);
		}
		if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] != NULL)
		{

			attron(COLOR_PAIR(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->col));
			if(dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->ch == '@')
			{
				if(hurt_this_turn)
				{
					attron(COLOR_PAIR(COLOR_RED));
				}
				else if(regen_this_turn)
				{
					attron(COLOR_PAIR(COLOR_GREEN));
				}
			}
			mvaddch(y,x,dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)]->ch);
			attron(COLOR_PAIR(COLOR_WHITE));
		}
		else if(dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
		{
			attron(COLOR_PAIR(COLOR_YELLOW));
			mvaddch(y,x,'^');
			attron(COLOR_PAIR(COLOR_WHITE));
		}
		else
		{
			mvaddch(y,x,terrains::terrain_char( dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].t));
		}
		attroff(A_BOLD);
}

void flash_number(int value,int x,int y,int colour,int source_x,int source_y)
{
	if(animation_delay == 0) return;
	if(x != source_x || y != source_y)
	{
		attron(COLOR_PAIR(COLOR_CYAN));
		mvaddch(source_y,source_x,'&');
	}
	int offset = 0;
	if(value >= 10)
		offset = 1;
	else if(value >= 100)
		offset = 2;
	else if(value >= 1000)
		offset = 3;
	int x_start = x - offset;
	if(x_start < 0)
	{
		x_start = 0;
	}
	attron(A_BOLD);
	attron(COLOR_PAIR(colour));
	mvprintw(y,x_start,"%d",value);

	attroff(A_BOLD);
	attron(COLOR_PAIR(COLOR_WHITE));
	refresh();
	mySleep(animation_delay);
	for(int xx = x_start; xx<=x_start+offset;xx++)
	{
		draw_tile_normal(xx,y);
	}
	if(x != source_x || y != source_y)
		draw_tile_normal(source_x,source_y);
	refresh();
}

void draw_dungeon()
{
	for(int y=0; y<dungeon::MAP_HEIGHT;y++)
	{
		for(int x=0; x<dungeon::MAP_WIDTH;x++)
		{
			draw_tile_normal(x,y);
		}
	}
	refresh();
}

void draw_pattern(patterns::pattern p, int x, int y,char ch)
{
	switch(ch)
	{
	case '@':
		attron(COLOR_PAIR(COLOR_YELLOW));
		break;
	case 'a':
		attron(COLOR_PAIR(COLOR_BLUE));
		break;
	case 'b': case 'x':
		attron(COLOR_PAIR(COLOR_CYAN));
		break;
	case 'c':
		attron(COLOR_PAIR(COLOR_GREEN));
		break;
	case 'd':
		attron(COLOR_PAIR(COLOR_MAGENTA));
		break;
	case 'E':
		attron(COLOR_PAIR(COLOR_RED));
		break;
	}
	mvaddch(y,x,ch);
	for(int i=0;i<8;i++)
	{
		int xx = x + movement::delta(i).first;
		int yy = y + movement::delta(i).second;
		mvaddch(yy,xx,terrains::terrain_class_char(p.p[i]));
	}
	attron(COLOR_PAIR(COLOR_WHITE));
}

void draw_bars(int offset_y,int offset_x)
{
	for(int i=0;i<3;i++)
	{
		mvaddch(offset_y+i,offset_x,'|');
	}
}

void draw_weapon(patterns::weapon* w,int offset, char ch,bool current_health=false)
{
	int offset_x = dungeon::MAP_WIDTH+1;
	int offset_y = offset;

	mvaddstr(offset_y,offset_x,w->user_pattern.rotations?"rot":"   ");
	mvaddstr(offset_y+2,offset_x,w->user_pattern.reflections?"rfl":"   ");
	draw_bars(offset_y,offset_x+3);
	draw_pattern(w->user_pattern,offset_x+5,offset_y+1,ch);
	draw_bars(offset_y,offset_x+7);

	mvaddstr(offset_y,offset_x+8,ch != '@'?"   ":(w->all_matching?"all":"one"));
	mvaddstr(offset_y+1,offset_x+8,w->must_be_adjacent?"adj":(w->must_be_visible?"vis":(ch=='@'?"all":"any")));
	mvaddstr(offset_y+2,offset_x+8,"   ");
	mvprintw(offset_y+2,offset_x+8,"%d",w->damage);

	draw_bars(offset_y,offset_x+11);
	draw_pattern(w->victim_pattern,offset_x+13,offset_y+1,ch=='@'?'x':'@');
	draw_bars(offset_y,offset_x+15);
	mvaddstr(offset_y,offset_x+16,w->victim_pattern.rotations?"rot":"   ");
	mvaddstr(offset_y+2,offset_x+16,w->victim_pattern.reflections?"rfl":"   ");

	mvaddstr(offset_y,offset_x+20,"   ");
	if(game_mode != 't' && game_mode != 'u')
		mvprintw(offset_y,offset_x+20,"%d",w->id);
	//mvaddstr(offset_y,offset_x+24,"weap#");
	switch(ch)
	{
	case '@':
		attron(COLOR_PAIR(COLOR_YELLOW));
		break;
	case 'a':
		attron(COLOR_PAIR(COLOR_BLUE));
		break;
	case 'b':
		attron(COLOR_PAIR(COLOR_CYAN));
		break;
	case 'c':
		attron(COLOR_PAIR(COLOR_GREEN));
		break;
	case 'd':
		attron(COLOR_PAIR(COLOR_MAGENTA));
		break;
	case 'E':
		attron(COLOR_PAIR(COLOR_RED));
		break;
	}
	mvaddch(offset_y+1,offset_x+20,ch);
	attron(COLOR_PAIR(COLOR_WHITE));
	mvaddstr(offset_y+2,offset_x+20,"    ");
	int health;
	if(current_health)
		health = p_ptr->health;
	else switch(ch)
	{
	case '@':
		health = player_health_max(dungeon_level); break;
	case 'a':
		health = monster_health(0); break;
	case 'b': case 'c': case 'd':
		health = monster_health(1); break;
	case 'E':
		health = monster_health(4);
		mvaddstr(offset_y,offset_x+24,"wepn");
		mvaddstr(offset_y+2,offset_x+24,"hlth");
		break;
	}
	if(ch != '@')
	{
		mvprintw(offset_y+2,offset_x+20,"%d",health);
	}
	//mvaddstr(offset_y+2,offset_x+25,ch!='@'?"HP":"  ");
}

void clear_area(int x,int y,int xr,int yr)
{
	for(int i=y;i<y+yr;i++)
		mvaddstr(i,x,std::string(xr,' ').c_str());
}

void do_options_menu_disp()
{
	clear_area(0,0,80,30);
	mvprintw(6,6,"Current animation delay: %d",animation_delay);
	mvprintw(7,6,"Press D to change animation delay.");
	mvprintw(8,6,"Press space to return.");
	refresh();
}

void do_options_menu()
{
	// Current animation delay: %d
	// Press D to change animation delay.
	// Press space to return.
	bool stop = false;
	while(!stop)
	{
		do_options_menu_disp();
		int kp;
		for(kp = getch(); !(kp==' '||kp=='d'||kp=='D'); kp=getch())
		{
		}
		switch(kp)
		{
		case ' ':
			stop = true; break;
		case 'd': case 'D':
			mvprintw(10,6,"Enter new animation delay (milliseconds, prev. value = %d):",animation_delay);
			mvaddstr(11,6,"Enter 0 to disable damage animations. Max 1000");
			refresh();
			int new_anim_delay=-1;
			while(new_anim_delay <0 || new_anim_delay > 1000)
			{
				move(12,12);
				echo();
				scanw("%d",&new_anim_delay);
				noecho();
				do_options_menu_disp();
				mvprintw(10,6,"Enter new animation delay (milliseconds, prev. value = %d):",animation_delay);
				mvaddstr(11,6,"Enter 0 to disable damage animations. Max 1000");
				refresh();
			}
			animation_delay = new_anim_delay;
			break;
		}
	}
}

void display_tutorial_text()
{
	int p = 1;
	int kp;
	bool stop = false;
	while(!stop)
	{
		if(dungeon_level == 1)
		{
			if(p == 1)
			{
				clear_area(3,3,44,24);
				mvaddstr(4,4,"Welcome to the tutorial!");
				mvaddstr(6,4,"You'll notice a few strange jumbles of");
				mvaddstr(7,4,"symbols on the right of the display.");
				mvaddstr(8,4,"These are weapons. The top one is yours,");
				mvaddstr(9,4,"and the lower one is your opponent's.");
				mvaddstr(11,4,"In this game you can't just attack any");
				mvaddstr(12,4,"time; you have to be standing in a");
				mvaddstr(13,4,"location that matches your weapon's");
				mvaddstr(14,4,"\"user pattern\". That's the one in the");
				mvaddstr(15,4,"top left, with the '@' in the middle.");
				mvaddstr(17,4,"Even when you're standing in the right");
				mvaddstr(18,4,"place, you can only hurt the enemy when");
				mvaddstr(19,4,"they stand in the wrong place. They have");
				mvaddstr(20,4,"to match the \"target pattern\" on the");
				mvaddstr(21,4,"right, which has the 'x' in the middle.");
				mvaddstr(23,4,"Your opponents have a similar weapon.");
				mvaddstr(25,4,"Press space to see page 2.");
			
				for(kp = getch(); kp != ' '; kp=getch())
				{}
				p = 2;

			}
			else if(p == 2)
			{
				clear_area(3,3,44,24);
				mvaddstr(4,4, "You and your enemies can only attack");
				mvaddstr(5,4, "or be attacked if you both stand in");
				mvaddstr(6,4, "locations that match the patterns.");
				mvaddstr(8,4, "The enemy has some blanks in its");
				mvaddstr(9,4, "\"user pattern\". These can match");
				mvaddstr(10,4,"both a wall '#' and a floor '.'.");
				mvaddstr(12,4,"All the patterns on this level can");
				mvaddstr(13,4,"be rotated, so your user pattern matches");
				mvaddstr(14,4,"both of these:");
				mvaddstr(15,4,"#.#   .#.   ");
				mvaddstr(16,4,".@.   #@#   You can press R to see where");
				mvaddstr(17,4,"#.#   .#.   your weapon matches. To see");
				mvaddstr(18,4,"the colour code for this feature, press");
				mvaddstr(19,4,"? for in-game help, or load the readme.");
				mvaddstr(21,4,"                QWE    YKU    789");
				mvaddstr(22,4," MOVE CONTROLS: ASD or H.L or 456");
				mvaddstr(23,4,"                ZXC    BJN    123");
				mvaddstr(25,4,"Press space to see page 3.");
			
				for(kp = getch(); kp != ' '; kp=getch())
				{}
				p = 3;
			}
			else
			{
				clear_area(3,3,44,24);
				mvaddstr(4,4,"The enemy's user pattern matches ");
				mvaddstr(5,4,"these and more: .#. #.. #.# #.#");
				mvaddstr(6,4,"                .b. .b. .b. .b.");
				mvaddstr(7,4,"                .#. ..# #.. #.#");
				mvaddstr(8,4,"The word \"adj\" (adjacent) in your weapon");
				mvaddstr(9,4,"means that you can only attack enemies");
				mvaddstr(10,4,"who are next to you.");
				mvaddstr(12,4,"The word \"one\" means you attack only");
				mvaddstr(13,4,"one enemy each turn, chosen at random.");
				mvaddstr(14,4,"You always attack automatically at the");
				mvaddstr(15,4,"end of every turn, and your enemies do");
				mvaddstr(16,4,"likewise.");
				mvaddstr(18,4,"You're going to be attacked by some 'b's.");
				mvaddstr(19,4,"They're quite stupid, so you'll be safe");
				mvaddstr(20,4,"if you're careful.");
				mvaddstr(21,4,"Now try to kill 10 'b's.");
				mvaddstr(23,4,"Stand on a ? to see these pages again.");
				mvaddstr(24,4,"Press S to start playing.");
				mvaddstr(25,4,"Press space to see page 1 again.");
				for(kp = getch(); kp != ' ' && kp != 's' && kp != 'S'; kp=getch())
				{}
				switch(kp)
				{
				case ' ':
					p = 1; break;
				case 's': case 'S':
					stop = true;
				}
			}
		}
		else
		{
if(p == 1)
			{
				clear_area(3,3,44,24);
				mvaddstr(4,4,"Level 2 will introduce some advanced");
				mvaddstr(5,4,"elements of the game.");

				mvaddstr(6,4,"There are two weapons on the ground,");
				mvaddstr(7,4,"represented by yellow ^s.");
				mvaddstr(9,4,"You can swap your current weapon for");
				mvaddstr(10,4,"one of those by standing on it and");
				mvaddstr(11,4,"pressing F.");
				mvaddstr(13,4,"One of the weapons has visual range, \"vis\",");
				mvaddstr(14,4,"so that you don't need to be adjacent");
				mvaddstr(15,4,"to an enemy to hurt them. The other");
				mvaddstr(16,4,"can hurt any number of enemies at once.");
				mvaddstr(18,4,"Killing 'b's isn't important. To win the");
				mvaddstr(19,4,"game you need to kill the bosses, 'E'.");
				mvaddstr(21,4,"There is one boss on this level.");
				mvaddstr(23,4,"     Kill it.");
				mvaddstr(25,4,"Press space to see page 2.");
			
				for(kp = getch(); kp != ' '; kp=getch())
				{}
				p = 2;

			}
			else
			{
				clear_area(3,3,44,24);
				mvaddstr(4,4, "Notes:");
				mvaddstr(6,4, "This level of the tutorial can be");
				mvaddstr(7,4, "difficult. Don't worry about trying to");
				mvaddstr(8,4, "beat it, rather use it to practice");
				mvaddstr(9,4,"tactics, pattern matching, and choosing");
				mvaddstr(10,4,"weapons according to the sitution.");
				mvaddstr(12,4,"In the real game, you have two weapons.");
				mvaddstr(13,4,"You can press F and G to swap the first");
				mvaddstr(14,4,"and second weapons, and you can press");
				mvaddstr(15,4,"R and T to recolour the map for the ");
				mvaddstr(16,4,"first and second weapons.");
				mvaddstr(18,4,"There are some numbered tiles on this");
				mvaddstr(19,4,"map. Anyone who stands on a numbered");
				mvaddstr(20,4,"tile will gradually rebuild the map");
				mvaddstr(21,4,"to give their weapon an advantage.");
				mvaddstr(23,4,"Stand on a ? to see these pages again.");
				mvaddstr(24,4,"Press S to start playing.");
				mvaddstr(25,4,"Press space to see page 1 again.");
			
				for(kp = getch(); kp != ' ' && kp != 's' && kp != 'S'; kp=getch())
				{}
				switch(kp)
				{
				case ' ':
					p = 1; break;
				case 's': case 'S':
					stop = true;
				}
			}
		}
	}
	
	//getch();
}

void do_help_page_disp(int p)
{
	clear_area(0,0,80,30);
	switch(p)
	{
	case 0:// 1: What am I trying to do here?
		mvaddstr(1,6,"1: What am I trying to do here?");
		mvaddstr(3,3,"Your objective is to survive 16 levels (in standard mode),");
		mvaddstr(4,3,"and kill all the bosses ("); attron(A_BOLD); attron(COLOR_PAIR(COLOR_RED)); addch('E'); attron(COLOR_PAIR(COLOR_WHITE)); attroff(A_BOLD); addstr(") on each level.");
		mvaddstr(5,3,"You can only deal damage with your weapons if the terrain surrounding you");
		mvaddstr(6,3,"and your target meets certain conditions, which depend on your weapons.");
		mvaddstr(7,3,"Your foes (");
		attron(A_BOLD);
		attron(COLOR_PAIR(COLOR_BLUE)); addch('a');
		attron(COLOR_PAIR(COLOR_CYAN)); addch('b');
		attron(COLOR_PAIR(COLOR_GREEN)); addch('c');
		attron(COLOR_PAIR(COLOR_MAGENTA)); addch('d');
		attron(COLOR_PAIR(COLOR_RED)); addch('E');
		attron(COLOR_PAIR(COLOR_WHITE));
		attroff(A_BOLD);
		addstr(") have the same constraints, and you will have to");
		mvaddstr(8,3,"act carefully to overcome them and win the game.");
		mvaddstr(9,3,"You have some advantages: you can wield two weapons, and you can choose");
		mvaddstr(10,3,"which weapons ("); attron(A_BOLD); attron(COLOR_PAIR(COLOR_YELLOW)); addch('^'); attron(COLOR_PAIR(COLOR_WHITE)); attroff(A_BOLD); addstr(") to use from those scattered on the floor");
		mvaddstr(11,3,"You're also much smarter than your opponents, but they will work together");
		mvaddstr(12,3,"against you.");
		mvaddstr(13,3,"Watch out, because certain tiles let you rebuild the dungeon so that your");
		mvaddstr(14,3,"weapon matches more locations. Both you and your enemies can use these."); 
		break;
	case 1:// 2: How do I do that?
		mvaddstr(1,6,"2: How do I do that?");
		mvaddstr(3,3,"There are few controls to learn.");
		mvaddstr(4,3,"Each turn you can move in one of 8 directions (orthogonal");
		mvaddstr(5,3,"or diagonal), or rest. After taking your turn, you will automatically");
		mvaddstr(6,3,"attack if possible. There are three options for controling movement:");
		 mvaddstr(7,3,"7   8   9   q   w   e   y   k   u");
		 mvaddstr(8,3," \\  |  /     \\  |  /     \\  |  /");
		 mvaddstr(9,3,"  \\ | /       \\ | /       \\ | /");
		mvaddstr(10,3,"4-- 5 --6   a-- s --d   h-- . --l");
		mvaddstr(11,3,"  / | \\       / | \\       / | \\");
		mvaddstr(12,3," /  |  \\     /  |  \\     /  |  \\");
		mvaddstr(13,3,"1   2   3   z   x   c   b   j   n");
		mvaddstr(14,3,"There are 5 other commands to learn.");
		mvaddstr(15,3,"Press F or G to pick up a weapon from the floor.");
		mvaddstr(16,3,"Press R or T to recolour the map to show locations that match your weapons.");
		mvaddstr(17,3,"Press ? to access this help/options menu, quit, or start a new game.");
		break;
	case 2:// 3: What do I see around me?
		mvaddstr(1,6,"3: What do I see around me?");
		attron(A_BOLD);
		mvaddch(3,3,'.');
		attroff(A_BOLD);
		mvaddstr(3,8,"Empty floor tile");
		attron(A_BOLD);
		mvaddch(4,3,'#');
		attroff(A_BOLD);
		mvaddstr(4,8,"Unpassable wall tile");
		attron(A_BOLD);
		mvaddch(5,3,'5');
		attroff(A_BOLD);
		mvaddstr(5,8,"Dungeon rebuilding button (any numeral) - stand on to use");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_YELLOW));mvaddch(6,3,'^');attron(COLOR_PAIR(COLOR_WHITE));attroff(A_BOLD);
		mvaddstr(6,8,"Weapon - Stand on and press f or g to swap with your current weapon");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_YELLOW));
		mvaddch(7,3,'@');
		attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(7,8,"Yourself. Move with qweasdzxc, hjklyubn, or numpad.");
		attron(A_BOLD);
		attron(COLOR_PAIR(COLOR_BLUE));mvaddch(8,3,'a');
		attron(COLOR_PAIR(COLOR_CYAN)); addch('b');
		attron(COLOR_PAIR(COLOR_GREEN)); addch('c');
		attron(COLOR_PAIR(COLOR_MAGENTA)); addch('d');
		attron(COLOR_PAIR(COLOR_WHITE));
		attroff(A_BOLD);
		mvaddstr(8,8,"Minor enemies. You don't need to kill any of them, but they can be");
		mvaddstr(9,8,"dangerous. Some of them are smarter than they look!");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_RED)); mvaddch(10,3,'E');attron(COLOR_PAIR(COLOR_WHITE));attroff(A_BOLD);
		mvaddstr(10,8,"Boss. Kill it!");
		mvaddstr(12,3,"You can view the patterns matched by your weapon and your enemies' weapons");
		mvaddstr(13,3,"in the display on the right hand side of the map. To use a weapon,");
		mvaddstr(14,3,"The weapon's user must match the pattern on the left. To be hit");
		mvaddstr(15,3,"by a weapon, the target must match the pattern on the right.");
		break;
	case 3:// 4: What is trying to stop me?
		mvaddstr(1,6,"4: What is trying to stop me?");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_BLUE));mvaddch(3,3,'a');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(3,8,"Very weak enemies. Low health, very bad weapons.");
		mvaddstr(4,8,"Will try to block squares you could safely stay on.");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_CYAN));mvaddch(5,3,'b');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(5,8,"Weak enemies. Medium health, bad weapons.");
		mvaddstr(6,8,"Will take the shortest path to you, no matter whether it can hurt you.");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_GREEN));mvaddch(7,3,'c');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(7,8,"Medium enemies. Medium health, OK weapons.");
		mvaddstr(8,8,"Will try to block squares you can use your weapon from.");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_MAGENTA));mvaddch(9,3,'d');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(9,8,"Dangerous enemies. Medium health, good weapons.");
		mvaddstr(10,8,"Will try to find squares it can attack you from.");
		attron(A_BOLD);attron(COLOR_PAIR(COLOR_RED));mvaddch(11,3,'E');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));
		mvaddstr(11,8,"Boss. High health, best weapons.");
		mvaddstr(12,8,"Will try to run away from you as best it can.");
		mvaddstr(14,3,"Some enemies have different objectives to the rest of the herd.");
		mvaddstr(15,3,"Occasionally an enemy might decide to go for a dungeon rebuilding button");
		mvaddstr(16,3,"to give its kind an advantage until the map changes.");
		mvaddstr(17,3,"Occasionally an enemy might decide to behave like a ");attron(A_BOLD);attron(COLOR_PAIR(COLOR_MAGENTA));addch('d');attroff(A_BOLD);attron(COLOR_PAIR(COLOR_WHITE));addstr(" and find");
		mvaddstr(18,3,"somewhere to hurt you from.");
		mvaddstr(19,3,"These smart enemies don't look any different until they start");
		mvaddstr(20,3,"accomplishing their goals, when they become capital letters.");
		mvaddstr(21,3,"Until then, you might notice them behaving differently to the others.");
		break;
	case 4:// 5: How do weapons work?
		mvaddstr(1,6,"5: How do weapons work?");
		mvaddstr(3,3,"Everyone tries to use their weapon immediately after taking their turn.");
		mvaddstr(4,3,"You have two weapons, which you try to use one after the other.");
		mvaddstr(5,3,"Every weapon (there are 100 weapons in a standard game) has two patterns:");
		mvaddstr(6,3,"A user pattern and a target pattern.");
		mvaddstr(7,3,"In order to use a weapon, the weapon's user pattern must match");
		mvaddstr(8,3,"the terrain the weapon's user is standing in.");
		mvaddstr(9,3,"In order for an enemy to be hit by a weapon, the enemy must be standing");
		mvaddstr(10,3,"in a location which matches the weapon's target pattern.");
		mvaddstr(11,3,"A weapon may affect either all enemies in range, or one chosen at random.");
		mvaddstr(12,3,"Some weapons require the user and target to be adjacent.");
		mvaddstr(13,3,"Some weapons require the user to have visual contact with the target.");
		mvaddstr(14,3,"Some weapons can affect any tile on the map from anywhere else.");
		mvaddstr(15,3,"A weapon does the same damage every time it hits.");
		mvaddstr(16,3,"On later levels, having a high-damage weapon can be important for survival.");
		break;
	case 5:// 6: How does pattern matching work? (Part 1)
		mvaddstr(1,6,"6: How does pattern matching work? (Part 1)");
		mvaddstr(3,3,"A pattern could be something like:");
		mvaddstr(4,3,"|.#.|..#|...|");
		mvaddstr(5,3,"|#@#| @ |.@.|");
		mvaddstr(6,3,"|   |   |...|");
		mvaddstr(7,3,"The player, @, is at the centre of these three patterns.");
		mvaddstr(8,3,"The player's position matches the pattern only if every . in the pattern has");
		mvaddstr(9,3,"a corresponding . in that position on the map, and likewise for every #.");
		mvaddstr(10,3,"Numbered squares match . because they are walkable. # is not walkable.");
		mvaddstr(11,3,"A blank space in a pattern can match anything.");
		mvaddstr(12,3,"Many weapons can also match 8 rotations and 8 more reflections of a pattern.");
		mvaddstr(13,3,"As example, here is a pattern:");
		mvaddstr(14,3,"|.#.|");
		mvaddstr(15,3,"|#@#|");
		mvaddstr(16,3,"|#..|");
		mvaddstr(17,3,"Here are all 8 of its rotations:");
		mvaddstr(18,3,"|.#.|#.#|.#.|#..|..#|.##|##.|#.#|");
		mvaddstr(19,3,"|#@#|.@.|#@.|.@#|#@#|.@.|.@#|#@.|");
		mvaddstr(20,3,"|#..|##.|.##|#.#|.#.|#.#|.#.|..#|");
		mvaddstr(21,3,"Here are all 8 of its reflections:");
		mvaddstr(22,3,"|.#.|#.#|.#.|..#|#..|##.|.##|#.#|");
		mvaddstr(23,3,"|#@#|.@.|.@#|#@.|#@#|.@.|#@.|.@#|");
		mvaddstr(24,3,"|..#|.##|##.|#.#|.#.|#.#|.#.|#..|");
		mvaddstr(25,3,"In general, it's best to have weapons with a lot of blanks in its patterns.");
		mvaddstr(26,3,"It's also best to have weapons that allow rotations and reflections.");
		mvaddstr(27,3,"That way, more of the locations on the level can be used by you to attack,");
		mvaddstr(28,3,"and more of the locations on the level are unsafe for your enemies.");
		break;
	case 6:// 7: How does pattern matching work? (Part 2)
		mvaddstr(1,6,"7: How does pattern matching work? (Part 2)");
		mvaddstr(3,3,"Sample weapon display and explanation:");
		mvaddstr(4,3,"\"rot|   |   |  #|rot 35\"");
		mvaddstr(5,3,"\"   |#E |adj| @ |    E \"");
		mvaddstr(6,3,"\"rfl|   |8  |   |    49\"");
		mvaddstr(7,3,"\"rot\" on the left and right means that both  patterns can be rotated.");
		mvaddstr(8,3,"\"rfl\" on the left means that the left (not right) pattern can be reflected.");
		mvaddstr(9,3,"\"E\" in the left pattern (the user pattern) means that bosses use this weapon.");

		mvaddstr(10,3,"\"adj\" means it only targets adjacent enemies. If it targeted visible enemies");
		mvaddstr(11,3,"or enemies anywhere on the map, it would say \"vis\" or \"all\"");
		mvaddstr(12,3,"\"8\" in the middle means the weapon does 8 damage.");

		mvaddstr(13,3,"\"@\" in the right (target) pattern means bosses can harm you in that position.");

		mvaddstr(14,3,"\"35\" at the top right means this weapon is #35. The best weapon is #99.");
		mvaddstr(15,3,"\"E\" on the right reminds you that this line describes this level's bosses.");
		mvaddstr(16,3,"\"49\" on the bottom right means that the bosses have 49 health.");

		mvaddstr(17,3,"In summary, this means that bosses can hit you for 8 damage, to do so they");
		mvaddstr(18,3,"must be adjacent to you, and in addition you must both have an adjacent wall");
		mvaddstr(19,3,"(in any direction, because of the rotations). If this were the player's");

		mvaddstr(20,3,"weapon, the middle section would have a top line, saying \"one\" or \"all\".");
		mvaddstr(21,3,"\"one\" means the weapon randomly chooses one of the available targets.");
		mvaddstr(22,3,"If it harmed all vulnerable enemies, it would say \"all\".");
		break;
	case 7:// 8: How do I use the colour map overlay?
		mvaddstr(1,6,"8: How do I use the colour map overlay?");
		mvaddstr(3,3,"Press 'r' to activate display assistance for your first weapon, or 't'");
		mvaddstr(4,3,"to activate it for your second  weapon. This will use colour to indicate");
		mvaddstr(5,3,"which tiles are valid to attack from and which tiles can be attacked.");
		mvaddstr(6,3,"On pressing 'r' or 't' tiles are coloured according to:");
		mvaddstr(7,3,"|------------------------|-------------|-------------|-------------|");
		mvaddstr(8,3,"|\\   Player can attack   |  Yes, from  |  No, maybe  |   No, not   |");
		mvaddstr(9,3,"| \\     that tile?       |  player's   |    from     |  from any   |");
		mvaddstr(10,3,"|  --------------------\\ |   current   |   another   |  position   |");
		mvaddstr(11,3,"|  Player can attack    \\|  position   |   position  |             |");
		mvaddstr(12,3,"|   from that tile?      |             |             |             |");
		mvaddstr(13,3,"|------------------------|-------------|-------------|-------------|");
		mvaddstr(14,3,"|         Yes            |   Green     |    Cyan     |    Blue     |");
		mvaddstr(15,3,"|------------------------|-------------|-------------|-------------|");
		mvaddstr(16,3,"|          No            |    Red      |   Magenta   |    White    |");
		mvaddstr(17,3,"|------------------------|-------------|-------------|-------------|");
		mvaddstr(18,3,"Using these commands doesn't take your turn, but the map colours");
		mvaddstr(19,3,"return to normal after you make your move.");
		break;
	case 8:// 9: Could you give some general advice?
		mvaddstr(1,6,"9: Could you give some general advice?");
		mvaddstr(3,3,"You don't have many health points and you can't afford to waste them.");
		mvaddstr(4,3,"Try to get weapons with enough damage to kill a,b,c, and d in one shot.");
		mvaddstr(5,3,"In general, weapons with more blanks in their pattern match more locations,");
		mvaddstr(6,3,"which means they can be used more flexibly. This concern must be balanced");
		mvaddstr(7,3,"with the need for enough damage to kill things quickly, and the utility");
		mvaddstr(8,3,"of weapons which can hit more than one enemy or can hit from a distance.");
		mvaddstr(9,3,"Try to get the first attack on an enemy. If an enemy is two squares away,");
		mvaddstr(10,3,"you might want to rest one turn so that you can move next to them.");
		mvaddstr(11,3,"Take a good look at your enemies' weapons before starting to play a level.");
		mvaddstr(12,3,"If a boss is trapped behind a wall and there's no route to it,");
		mvaddstr(13,3,"You can free it by using the dungeon rebuilding buttons (numbers).");
		mvaddstr(14,3,"Try to look at all the floor weapons on each level;");
		mvaddstr(15,3,"some of them might be very good for their level.");
		break;
	}
	mvaddstr(29,3,"Press space to return, or left/right to view another page.");
	refresh();
}

void do_help_page(int p)
{
	int actual_page = p;
	bool stop = false;
	while(!stop)
	{
		do_help_page_disp(actual_page);
		int kp;
		for(kp = getch(); !(kp==' '||kp==KEY_LEFT||kp==KEY_RIGHT); kp=getch())
		{
		}
		switch(kp)
		{
		case ' ':
			stop = true;
			break;
		case KEY_LEFT:
			actual_page = (actual_page+8)%9;
			break;
		case KEY_RIGHT:
			actual_page = (actual_page+1)%9;
			break;
		}
	}
}

void do_help_menu_disp()
{
	clear_area(0,0,80,30);
	mvaddstr(6,6,"Press a number to see a help page.");
	mvaddstr(7,6,"1: What am I trying to do here?");
	mvaddstr(8,6,"2: How do I do that?");
	mvaddstr(9,6,"3: What do I see around me?");
	mvaddstr(10,6,"4: What is trying to stop me?");
	mvaddstr(11,6,"5: How do weapons work?");
	mvaddstr(12,6,"6: How does pattern matching work? (Part 1)");
	mvaddstr(13,6,"7: How does pattern matching work? (Part 2)");
	mvaddstr(14,6,"8: How do I use the colour map overlay?");
	mvaddstr(15,6,"9: Could you give some general advice?");
	mvaddstr(16,6,"Press space to go back.");
	refresh();

}
void do_help_menu()
{
	int kp = 0;
	while(kp != ' ')
	{
		do_help_menu_disp();
		for(kp = getch(); !(('1' <= kp && '9' >= kp)||kp==' '); kp=getch())
		{
		}
		if('1' <= kp && '9' >= kp)
		{
			do_help_page(kp-'1');
		}
	}
}

void do_new_game_menu_disp()
{
	clear_area(0,0,80,30);
	mvaddstr(6,6,"Press S to play a Standard game.");
	mvaddstr(7,6,"  (16 levels, 100 weapons)");
	mvaddstr(9,6,"Press T to play the Tutorial.");
	mvaddstr(10,6,"  (highly recommended)");
	mvaddstr(12,6,"Press Q to play a Quick game.");
	mvaddstr(13,6,"  (8 levels, fewer bosses, more floor weapons)");
	mvaddstr(15,6,"Press B to play a Blacksmith Game.");
	mvaddstr(16,6,"  (10 levels, high-quality weapons for everyone)");
	mvaddstr(18,6,"Press U to play an Unlimited Game.");
	mvaddstr(19,6,"  (Unbounded levels, random weapons each level)");
	mvaddstr(21,6,"Press M to play a Munchkin Game.");
	mvaddstr(22,6,"  (Same as Standard, with the best weapons available sooner)");
	mvaddstr(24,6,"Press space to return.");
	refresh();
}

void do_new_game_menu()
{
	do_new_game_menu_disp();
	int kp;
	for(kp = getch(); !(kp=='S'||kp=='s'||kp=='T'||kp=='t'||kp=='Q'||kp=='q'||kp=='B'||kp=='b'||kp=='U'||kp=='u'||kp=='M'||kp=='m'||kp==' '); kp=getch())
	{
	}
	switch(kp)
	{
	case 'S': case 's':
		game_mode = 's';
		break;
	case 'T': case 't':
		game_mode = 't';
		break;
	case 'Q': case 'q':
		game_mode = 'q';
		break;
	case 'B': case 'b':
		game_mode = 'b';
		break;
	case 'U': case 'u':
		game_mode = 'u';
		break;
	case 'M': case 'm':
		game_mode = 'm';
		break;
	case ' ':
		game_mode = 0;
		break;
	}

	// Press S to play a Standard game.
	//   (16 levels, 100 weapons)
	// Press T to play the Tutorial.
	//   (highly recommended)
	// Press Q to play a Quick game.
	//   (8 levels, fewer bosses, more floor weapons)
	// Press B to play a Blacksmith Game.
	//   (10 levels, high-quality weapons for everyone)
	// Press U to play an Unlimited Game.
	//   (Unbounded levels, random weapons each level)
	// Press M to play a Munchkin Game.
	//   (Same as Standard, with the best weapons available sooner)
	// Press space to return.

}

void do_pause_menu_disp()
{
	clear_area(0,0,80,30);
	mvaddstr(6,6,"Menu");
	mvaddstr(7,6,"Press O for options.");
	mvaddstr(8,6,"Press H for help.");
	mvaddstr(9,6,"Press S to suicide (and start a new game?).");
	mvaddstr(10,6,"Press Q to quit.");
	mvaddstr(11,6,"Press space to return.");
	refresh();
}

void do_pause_menu()
{
	bool stop = false;
	while(!stop)
	{
		do_pause_menu_disp();
		// Menu
		// Press O for options.
		// Press H for help.
		// Press S to suicide (and start a new game?).
		// Press Q to quit.
		// Press space to return.

		int kp;
		for(kp = getch(); !(kp=='Q'||kp=='q'||kp=='S'||kp=='s'||kp=='H'||kp=='h'||kp=='O'||kp=='o'||kp==' '); kp=getch())
		{
		}
		switch(kp)
		{
		case 'Q': case 'q':
			exit(0);
			break;
		case 'S': case 's':
			dungeon::do_damage(p_ptr,p_ptr,p_ptr->health);
			stop = true;
			break;
		case 'H': case 'h':
			do_help_menu();
			break;
		case 'O': case 'o':
			do_options_menu();
			break;
		case ' ':
			stop = true;
			break;
		}
	}
	draw_ui();
	draw_dungeon();
}

void do_start_menu_disp()
{
	clear_area(0,0,80,30);
	mvaddstr(6,6,"Welcome to Encircled!");
	mvaddstr(7,6,"Press S to start a new game.");
	mvaddstr(8,6,"Press O for options.");
	mvaddstr(9,6,"Press H for help.");
	mvaddstr(10,6,"Press Q to quit.");
	refresh();
}

void do_start_menu()
{
	bool stop = false;
	game_mode = 0;
	while(game_mode == 0)
	{
		// Welcome to Encircled!
		// Press S to start a new game.
		// Press O for options.
		// Press H for help.
		// Press Q to quit.
		do_start_menu_disp();
		int kp;
		for(kp = getch(); !(kp=='Q'||kp=='q'||kp=='S'||kp=='s'||kp=='H'||kp=='h'||kp=='O'||kp=='o'); kp=getch())
		{
		}
		switch(kp)
		{
		case 'Q': case 'q':
			exit(0);
			break;
		case 'S': case 's':
			do_new_game_menu();
			break;
		case 'H': case 'h':
			do_help_menu();
			break;
		case 'O': case 'o':
			do_options_menu();
			break;
		}
	}
}


void draw_ui()
{
	for(int y=0;y<dungeon::MAP_HEIGHT; y++)
	{
		mvaddch(y,dungeon::MAP_WIDTH,' ');
	}
	mvaddstr(0,dungeon::MAP_WIDTH+1,"Press ? for help and options");
	mvaddstr(1,dungeon::MAP_WIDTH+1,"                ");
	if(hurt_this_turn)
	{
		attron(COLOR_PAIR(COLOR_RED));
		attron(A_BOLD);
	}
	else if(regen_this_turn)
	{
		attron(COLOR_PAIR(COLOR_GREEN));
		attron(A_BOLD);
	}
	mvprintw(1,dungeon::MAP_WIDTH+1,"HP %d/%d",p_ptr->health,player_health_max(dungeon_level));
	if(hurt_this_turn || regen_this_turn)
	{
		attron(COLOR_PAIR(COLOR_WHITE));
		attroff(A_BOLD);
	}
	if(game_mode != 't' || dungeon_level != 1)
	{
		mvaddstr(1,dungeon::MAP_WIDTH+12,"Bosses left:      ");
		mvprintw(1,dungeon::MAP_WIDTH+25,"%d/%d",boss_quota- bosses_killed,boss_quota);
	}
	mvaddstr(2,dungeon::MAP_WIDTH+1,"Player weapons:");
	draw_weapon(p_ptr->my_weapon,3,'@');
	if(game_mode != 't')
	{
		draw_weapon(p_ptr->my_other_weapon,7,'@',true);
	}
	mvaddstr(10,dungeon::MAP_WIDTH+1,"Enemies this level:");
	draw_weapon(patterns::weapons[patterns::monster_weapon_id(1)],15,'b');
	if(game_mode != 't')
	{
		draw_weapon(patterns::weapons[patterns::monster_weapon_id(0)],11,'a');
		draw_weapon(patterns::weapons[patterns::monster_weapon_id(2)],19,'c');
		draw_weapon(patterns::weapons[patterns::monster_weapon_id(3)],23,'d');
	}
	if(game_mode != 't' || dungeon_level == 2)
		draw_weapon(patterns::weapons[patterns::monster_weapon_id(4)],27,'E');
}


bool play_turn()
{
	for(unsigned int i=0; i<actors.size(); i++)
	{
		if(actors[i]->health <= 0)
		{
			actors.erase(actors.begin()+i);
			i--;
		}
	}
	for(unsigned int i=0; i<actors.size(); i++)
	{
		if(actors[i]->health <= 0) continue;
		
		if(actors[i]->is_player)
		{
			if(dungeon::dungeon_terrain[dungeon::dungeon_index(p_ptr->x,p_ptr->y)].t == terrains::HELP)
			{
				display_tutorial_text();
				draw_ui();
				draw_dungeon();
			}
			regen_this_turn = false;
			bool input_accepted = false;
			while(!input_accepted)
			{
				int kp = getch();
				switch(kp)
				{
				case '?': case '/':
					do_pause_menu();
					if(p_ptr->health == 0) input_accepted = true;
					break;
				case 'r': case 'R':
					draw_dungeon_player();
					break;
				//case 't': information would be delayed and inaccurate and unhelpful
				//	draw_dungeon_enemies();
				//	break;
				case 't': case 'T':
					if(game_mode != 't')
						draw_dungeon_player_other();
					break;
				/*case 'v':
					p_ptr->my_weapon = patterns::weapons[rand()%100];
					p_ptr->my_other_weapon = patterns::weapons[rand()%100];
					draw_ui();
					break;*/
				case '*':
					bosses_killed++;
					input_accepted = true;
					break;
				case 'f': case 'F':
					if(dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)] != NULL)
					{
						patterns::weapon* swap = p_ptr->my_weapon;
						p_ptr->my_weapon = dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];
						dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)] = swap;
						draw_ui();
						input_accepted = true;
					}
					break;
				case 'g': case 'G':
					if(dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)] != NULL && game_mode != 't')
					{
						patterns::weapon* swap = p_ptr->my_other_weapon;
						p_ptr->my_other_weapon = dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)];
						dungeon::dungeon_weapons[dungeon::dungeon_index(p_ptr->x,p_ptr->y)] = swap;
						draw_ui();
						input_accepted = true;
					}
					break;
				case '1': case 'b': case 'z': case 'B': case 'Z': case KEY_C1:
					input_accepted = movement::move_actor(0,p_ptr);
					break;
				case '2': case 'j': case 'x': case 'J': case 'X': case KEY_C2:
					input_accepted = movement::move_actor(1,p_ptr);
					break;
				case '3': case 'n': case 'c': case 'N': case 'C': case KEY_C3:
					input_accepted = movement::move_actor(2,p_ptr);
					break;
				case '4': case 'h': case 'a': case 'H': case 'A': case KEY_B1:
					input_accepted = movement::move_actor(7,p_ptr);
					break;
				case '5': case '.': case 's': case 'S': case KEY_B2:
					input_accepted = movement::move_actor(-1,p_ptr);
					break;
				case '6': case 'l': case 'd': case 'L': case 'D': case KEY_B3:
					input_accepted = movement::move_actor(3,p_ptr);
					break;
				case '7': case 'y': case 'q': case 'Y': case 'Q': case KEY_A1:
					input_accepted = movement::move_actor(6,p_ptr);
					break;
				case '8': case 'k': case 'w': case 'K': case 'W': case KEY_A2:
					input_accepted = movement::move_actor(5,p_ptr);
					break;
				case '9': case 'u': case 'e': case 'U': case 'E': case KEY_A3:
					input_accepted = movement::move_actor(4,p_ptr);
					break;

				}
			}
			patterns::note_all(true,-1);
			hurt_this_turn = false;
		}
		else
		{
			if(actors[i]->go_for_damage)
			{
				movement::damage_ai(actors[i]);
			}
			else if(actors[i]->go_for_restructuring)
			{
				movement::restructure_ai(actors[i]);
			}
			else switch(actors[i]->ch)
			{
			case 'a': case 'A':
				movement::block_safe_ai(actors[i]);
				break;
			case 'b': case 'B':
				movement::surround_player_ai(actors[i]);
				break;
			case 'c': case 'C':
				movement::prevent_attack_ai(actors[i]);
				break;
			case 'd': case 'D':
				movement::camp_damage_ai(actors[i]);
				break;
			case 'e': case 'E':
				movement::run_ai(actors[i]);
				break;
			}
		}
		patterns::use_weapon(actors[i]->my_weapon,actors[i]);
		if(game_mode != 't' && actors[i]->my_other_weapon != NULL)
			patterns::use_weapon(actors[i]->my_other_weapon,actors[i]);
	}
	dungeon::do_all_restructuring();
	movement::bfs();
	if(rand()%3)
		patterns::note_all(false,rand()%5);
	if(game_mode == 't' && dungeon_level == 1)
	{
		return p_ptr->health > 0 && enemies_killed[1] < 10;
	}
	return (p_ptr->health > 0 && bosses_killed < boss_quota);
}

namespace dungeon
{
	void blank_dungeon()
	{
		for(int x=0;x<MAP_WIDTH;x++)
		{
			for(int y=0;y<MAP_HEIGHT;y++)
			{
				dungeon_occupants[dungeon_index(x,y)] = NULL;
				dungeon::dungeon_terrain[dungeon_index(x,y)] = terrains::tt(terrains::FLOOR,false);
				dungeon::dungeon_weapons[dungeon_index(x,y)] = NULL;
			}
		}

	}

	void white_noise_area(int xx, int yy, int xr, int yr, int p, int q)
	{
		for(int x=xx;x<xx+xr;x++)
		{
			for(int y=yy;y<yy+yr;y++)
			{
				dungeon::dungeon_terrain[dungeon_index(x,y)] = terrains::tt(rand()%q<p?terrains::FLOOR : terrains::WALL,false);
				dungeon::generator_responsible[dungeon_index(x,y)] = 0;
			}
		}
	}

	void chessboard_area(int xx, int yy, int xr, int yr,int stripe_width)
	{
		int sgn = (rand()%2==0)?1:-1;
		int parity = rand()%stripe_width;
		for(int x=xx;x<xx+xr;x++)
		{
			for(int y=yy;y<yy+yr;y++)
			{
				dungeon::dungeon_terrain[dungeon_index(x,y)] = terrains::tt(((sgn*x+y+parity)%stripe_width)!=0?terrains::FLOOR : terrains::WALL,false);
				dungeon::generator_responsible[dungeon_index(x,y)] = 1;
			}
		}
	}

	void wrap_area(int xx, int yy, int xr, int yr, int wavelength)
	{
		int sgn = (rand()%2==0)?1:-1;
		int real_wavelength = wavelength;
		if(sgn==1)
		{
			while (yr%real_wavelength == 0)
				real_wavelength++;
		}
		else
		{
			while (xr%real_wavelength == 0)
				real_wavelength++;
		}
		int offset = rand()%wavelength;
		for(int x=xx;x<xx+xr;x++)
		{
			for(int y=yy;y<yy+yr;y++)
			{
				
				dungeon::dungeon_terrain[dungeon_index(x,y)] = terrains::tt((x*(sgn?1:yr)+y*(sgn?xr:1)+offset)%wavelength!=0?terrains::FLOOR : terrains::WALL,false);
				dungeon::generator_responsible[dungeon_index(x,y)] = 2;
			}
		}
	}

	void pillar_area(int xx, int yy, int xr, int yr, int pillar_size_x, int corridor_size_x, int pillar_size_y, int corridor_size_y)
	{
		int pattern_size_x = pillar_size_x + corridor_size_x;
		int pattern_size_y = pillar_size_y + corridor_size_y;
		int offset_x = rand()%pattern_size_x;
		int offset_y = rand()%pattern_size_y;
		for(int x=xx;x<xx+xr;x++)
		{
			for(int y=yy;y<yy+yr;y++)
			{
				dungeon::dungeon_terrain[dungeon_index(x,y)] = terrains::tt((((x + offset_x)% pattern_size_x < corridor_size_x) || ((y + offset_y)% pattern_size_y < corridor_size_y))?terrains::FLOOR : terrains::WALL,false);
				dungeon::generator_responsible[dungeon_index(x,y)] = 3;
			}
		}
	}
	void choice_area(int xx, int yy, int xr, int yr, bool top_level);

	void choice_area(int xx, int yy, int xr, int yr, bool top_level=false)
	{
		int r;
		if(top_level)
		{
			r = 5;
			if(!(xr > 10 && yr > 7))
				r = rand()%5;
		}
		else if(xr > 10 && yr > 7)
			r = rand()%11;
		else
			r = rand()%5;
		switch(r)
		{
		case 0:
			white_noise_area(xx,yy,xr,yr,65,100);
			break;
		case 1:
			chessboard_area(xx,yy,xr,yr,rand()%3+2);
			break;
		case 2:
			wrap_area(xx,yy,xr,yr,rand()%4+2);
			break;
		case 3: case 4:
			pillar_area(xx,yy,xr,yr,rand()%3+1,rand()%3+1,rand()%3+1,rand()%3+1);
			break;
		case 5: case 6: case 7: case 8: case 9: case 10:
			int xo_min = xr/5+1;
			int yo_min = yr/5+1;
			int xo = rand()%(xr-xo_min)+xo_min;
			int yo = rand()%(yr-yo_min)+yo_min;
			choice_area(xx,yy,xo,yo,top_level&&(rand()%3!=0));
			choice_area(xx+xo,yy,xr-xo,yo,top_level&&(rand()%3!=0));
			choice_area(xx,yy+yo,xo,yr-yo,top_level&&(rand()%3!=0));
			choice_area(xx+xo,yy+yo,xr-xo,yr-yo,top_level&&(rand()%3!=0));
			break;
		}
	}
	
	void init_dungeon(int level)
	{
		blank_dungeon();
		choice_area(0,0,MAP_WIDTH,MAP_HEIGHT,true);
		/*int r = rand()%10;
		switch(r)
		{
		case 0:
			white_noise_area(0,0,MAP_WIDTH,MAP_HEIGHT,65,100);
		case 1:
			chessboard_area(0,0,MAP_WIDTH,MAP_HEIGHT,rand()%3+2);
		case 2:
			wrap_area(0,0,MAP_WIDTH,MAP_HEIGHT,rand()%4+2);
		case 4:
			pillar_area(0,0,MAP_WIDTH,MAP_HEIGHT,rand()%3+1,rand()%3+1,rand()%3+1,rand()%3+1);
		case 5: case 6: case 7: case 8: case 9:
			choice_area(0,0,MAP_WIDTH,MAP_HEIGHT);
		}*/

		for(int i = 0;i<level/2+3; i++)
		{
			int x = rand()%MAP_WIDTH;
			int y = rand()%MAP_HEIGHT;
			while(dungeon::dungeon_terrain[dungeon_index(x,y)].t != terrains::FLOOR)
			{
				x = rand()%MAP_WIDTH;
				y = rand()%MAP_HEIGHT;
			}
			dungeon::dungeon_terrain[dungeon_index(x,y)].t = (terrains::terrain)(std::min(8,std::max(2,rand()%(1+level/2))));
			dungeon::dungeon_terrain[dungeon_index(x,y)].permanent = true;
		}
		switch(game_mode)
		{
		case 't':
			if(level==1)
			{
				chessboard_area(0,0,MAP_WIDTH/2,MAP_HEIGHT,2);
				pillar_area(MAP_WIDTH/2,0,MAP_WIDTH/2,MAP_HEIGHT,1,1,1,1);
			}
			else
			{
				dungeon::dungeon_weapons[dungeon::dungeon_index(MAP_WIDTH/3,MAP_HEIGHT/2)] = patterns::weapons[2];
				dungeon::dungeon_weapons[dungeon::dungeon_index(2*MAP_WIDTH/3,MAP_HEIGHT/2)] = patterns::weapons[3];
				dungeon::dungeon_terrain[dungeon::dungeon_index(MAP_WIDTH/3,MAP_HEIGHT/2)] = terrains::tt(terrains::FLOOR,true);
				dungeon::dungeon_terrain[dungeon::dungeon_index(2*MAP_WIDTH/3,MAP_HEIGHT/2)] = terrains::tt(terrains::FLOOR,true);
			}
			for(int ix=-1;ix<2;ix++)
			{
				for(int iy=-1;iy<2;iy++)
				{
					dungeon::dungeon_terrain[dungeon::dungeon_index(MAP_WIDTH/2+ix,MAP_HEIGHT/2+iy)] = terrains::tt(terrains::HELP,true);
					dungeon::dungeon_terrain[dungeon::dungeon_index(MAP_WIDTH/2+ix,MAP_HEIGHT/2+iy)] = terrains::tt(terrains::HELP,true);
				}
			}
			break;
		case 's':
			for(int i=(dungeon_level_number(level)-1)*6;i<(dungeon_level_number(level)-1)*6+10;i++)
			{
				int x = rand()%MAP_WIDTH;
				int y = rand()%MAP_HEIGHT;
				while(dungeon::dungeon_terrain[dungeon_index(x,y)].t != terrains::FLOOR || dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
				{
					x = rand()%MAP_WIDTH;
					y = rand()%MAP_HEIGHT;

				}
				if(!patterns::weapons[i]->must_be_adjacent)
					nonmelee_available = true;
				dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] = patterns::weapons[i];
				dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].permanent = true;
			}
			break;
		case 'm':
			for(int i=0;i<10;i++)
			{
				int x = rand()%MAP_WIDTH;
				int y = rand()%MAP_HEIGHT;
				while(dungeon::dungeon_terrain[dungeon_index(x,y)].t != terrains::FLOOR || dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
				{
					x = rand()%MAP_WIDTH;
					y = rand()%MAP_HEIGHT;

				}
				int r = rand()%100;
				if(!patterns::weapons[r]->must_be_adjacent)
					nonmelee_available = true;
				dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] = patterns::weapons[r];
				dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].permanent = true;
			}
			break;
		case 'q': case 'b':
			for(int i=(level-1)*10;i<(level)*10;i++)
			{
				int x = rand()%MAP_WIDTH;
				int y = rand()%MAP_HEIGHT;
				while(dungeon::dungeon_terrain[dungeon_index(x,y)].t != terrains::FLOOR || dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
				{
					x = rand()%MAP_WIDTH;
					y = rand()%MAP_HEIGHT;

				}
				if(!patterns::weapons[i]->must_be_adjacent)
					nonmelee_available = true;
				dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] = patterns::weapons[i];
				dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].permanent = true;
			}
			break;
		case 'u':
			if(level > 1)
			{
				patterns::weapon* w1 = p_ptr->my_weapon;
				patterns::weapon* w2 = p_ptr->my_other_weapon;
				for(int i=1;i<17;i++)
				{
					if(p_ptr->my_weapon != patterns::weapons[i] && p_ptr->my_other_weapon != patterns::weapons[i])
						delete patterns::weapons[i];
				}
				patterns::weapons.clear();
				patterns::weapons.push_back(w1);
				patterns::weapons.push_back(w2);
				
				for(int i=2;i<17;i++)
				{
					patterns::weapon* w=NULL;
					do
					{
						if(w!=NULL)
							delete w;
						w = patterns::generate_weapon();
					}
					while(
						((w->pu > 253 || w->pv > 253) && !w->must_be_adjacent) ||
						(w->pu < 60) ||
						(w->pv < 100 && (!w->all_matching || w->must_be_adjacent)) ||
						((!w->must_be_adjacent && !w->must_be_visible) && (w->pu > 129 && w->pv > 129))
						);
					patterns::weapons.push_back(w);
				}
				for(int i=0; i<17;i++)
				{
					patterns::weapons[i]->id = i;
				}
			}
			for(int i=7;i<17;i++)
			{
				int x = rand()%MAP_WIDTH;
				int y = rand()%MAP_HEIGHT;
				while(dungeon::dungeon_terrain[dungeon_index(x,y)].t != terrains::FLOOR || dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] != NULL)
				{
					x = rand()%MAP_WIDTH;
					y = rand()%MAP_HEIGHT;

				}
				if(!patterns::weapons[i]->must_be_adjacent)
					nonmelee_available = true;
				dungeon::dungeon_weapons[dungeon::dungeon_index(x,y)] = patterns::weapons[i];
				dungeon::dungeon_terrain[dungeon::dungeon_index(x,y)].permanent = true;
			}
			break;
		}
	}
}

void init_game()
{
	clear_area(0,0,80,30);
	for(int i=0;i<5;i++)
	{
		enemies_killed[i] = 0;
		damage_from_enemies[i] = 0;
	}
	switch(game_mode)
	{
	case 's': case 'm':
		last_level = 20; break;
	case 'q':
		last_level = 8; break;
	case 'b':
		last_level = 10; break;
	case 'u':
		last_level = -1; break;
	case 't':
		last_level = 2; break;
	}
	nonmelee_available = false;
	hurt_this_turn = false;
	regen_this_turn = false;
	patterns::weapons.clear();
	for(unsigned int i=0;i<patterns::weapons.size();i++)
	{
		delete patterns::weapons[i];
	}
	patterns::weapons.clear();
	for(unsigned int i=0;i<actors.size();i++)
	{
		delete actors[i];
	}
	actors.clear();
	switch(game_mode)
	{
	case 's': case 'm':
		{
			for(int i=0;i<100;i++)
			{
				patterns::weapon* w=NULL;
				do
				{
					if(w!=NULL)
						delete w;
					w = patterns::generate_weapon();
				}
				while(
					((w->pu > 253 || w->pv > 253) && !w->must_be_adjacent) ||
					(w->pu < 60) ||
					(w->pv < 100 && (!w->all_matching || w->must_be_adjacent)) ||
					((!w->must_be_adjacent && !w->must_be_visible) && (w->pu > 129 && w->pv > 129))
					);
				patterns::weapons.push_back(w);
			}
			std::sort(patterns::weapons.begin(),patterns::weapons.end(),patterns::lower_quality);
			for(int i=0;i<100;i++)
			{
				patterns::weapons[i]->id = i;
			}
			break;
		}
	case 'q':
		{
			for(int i=0;i<80;i++)
			{
				patterns::weapon* w=NULL;
				do
				{
					if(w!=NULL)
						delete w;
					w = patterns::generate_weapon();
				}
				while(
					((w->pu > 253 || w->pv > 253) && !w->must_be_adjacent) ||
					(w->pu < 60) ||
					(w->pv < 100 && (!w->all_matching || w->must_be_adjacent)) ||
					((!w->must_be_adjacent && !w->must_be_visible) && (w->pu > 129 && w->pv > 129))
					);
				patterns::weapons.push_back(w);
			}
			std::sort(patterns::weapons.begin(),patterns::weapons.end(),patterns::lower_quality);
			for(int i=0;i<80;i++)
			{
				patterns::weapons[i]->id = i;
			}
			break;
		}
	case 'b':
		{
			std::vector<patterns::weapon*> adj_one;
			std::vector<patterns::weapon*> vis_one;
			std::vector<patterns::weapon*> all_one;
			std::vector<patterns::weapon*> adj_all;
			std::vector<patterns::weapon*> vis_all;
			std::vector<patterns::weapon*> all_all;
			for(int i=0;i<200;i++)
			{
				patterns::weapon* w=NULL;
				do
				{
					if(w!=NULL)
						delete w;
					w = patterns::generate_weapon();
				}
				while(
					((w->pu > 253 || w->pv > 253) && !w->must_be_adjacent) ||
					(w->pu < 60) ||
					(w->pv < 100 && (!w->all_matching || w->must_be_adjacent)) ||
					((!w->must_be_adjacent && !w->must_be_visible) && (w->pu > 129 && w->pv > 129))
					);
				if(w->all_matching)
				{
					if(w->must_be_adjacent)
					{
						adj_all.push_back(w);
					}
					else if(w->must_be_visible)
					{
						vis_all.push_back(w);
					}
					else
					{
						all_all.push_back(w);
					}
				}
				else
				{
					if(w->must_be_adjacent)
					{
						adj_one.push_back(w);
					}
					else if(w->must_be_visible)
					{
						vis_one.push_back(w);
					}
					else
					{
						all_one.push_back(w);
					}
				}
			}
			std::sort(adj_one.begin(),adj_one.end(),patterns::lower_quality);
			std::sort(vis_one.begin(),vis_one.end(),patterns::lower_quality);
			std::sort(all_one.begin(),all_one.end(),patterns::lower_quality);
			std::sort(adj_all.begin(),adj_all.end(),patterns::lower_quality);
			std::sort(vis_all.begin(),vis_all.end(),patterns::lower_quality);
			std::sort(all_all.begin(),all_all.end(),patterns::lower_quality);
			for(int i=0;i<100;i++)
			{
				bool added_weapon = false;
				while(!added_weapon)
				{
					int r = rand()%6;
					switch(r)
					{
					case 0:
						if(adj_one.size() != 0)
						{
							patterns::weapons.push_back(adj_one.back());
							adj_one.pop_back();
							added_weapon = true;
						}
						break;
					case 1:
						if(vis_one.size() != 0)
						{
							patterns::weapons.push_back(vis_one.back());
							vis_one.pop_back();
							added_weapon = true;
						}
						break;
					case 2:
						if(all_one.size() != 0)
						{
							patterns::weapons.push_back(all_one.back());
							all_one.pop_back();
							added_weapon = true;
						}
						break;
					case 3:
						if(adj_all.size() != 0)
						{
							patterns::weapons.push_back(adj_all.back());
							adj_all.pop_back();
							added_weapon = true;
						}
						break;
					case 4:
						if(vis_all.size() != 0)
						{
							patterns::weapons.push_back(vis_all.back());
							vis_all.pop_back();
							added_weapon = true;
						}
						break;
					case 5:
						if(all_all.size() != 0)
						{
							patterns::weapons.push_back(all_all.back());
							all_all.pop_back();
							added_weapon = true;
						}
						break;
					}
				}
			}
			std::sort(patterns::weapons.begin(),patterns::weapons.end(),patterns::lower_quality);
			for(int i=0;i<100;i++)
			{
				patterns::weapons[i]->id = i;
			}
			break;
		}
		case 't':
			{
				for(int i=0;i<6;i++)
				{
					patterns::weapons.push_back(patterns::tutorial_weapon(i));
				}
				break;
			}
		case 'u':
			{
				for(int i=0;i<17;i++)
				{
					patterns::weapon* w=NULL;
					do
					{
						if(w!=NULL)
							delete w;
						w = patterns::generate_weapon();
					}
					while(
						((w->pu > 253 || w->pv > 253) && !w->must_be_adjacent) ||
						(w->pu < 60) ||
						(w->pv < 100 && (!w->all_matching || w->must_be_adjacent)) ||
						((!w->must_be_adjacent && !w->must_be_visible) && (w->pu > 129 && w->pv > 129))
						);
					patterns::weapons.push_back(w);
				}
				for(int i=0;i<17;i++)
				{
					patterns::weapons[i]->id = i;
				}
				break;
			}
	}
	dungeon_level = 1;
	dungeon::init_dungeon(1);
	while(true)
	{
		int x = rand()%dungeon::MAP_WIDTH;
		int y = rand()%dungeon::MAP_HEIGHT;
		if (dungeon::walkable(x,y))
		{
			p_ptr = new actor(x,y,'@',COLOR_YELLOW,player_health_max(1),true);
			dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] = p_ptr;
			actors.push_back(p_ptr);
			break;
		}
	}
	switch(game_mode)
	{
	case 's': case 'b':
		p_ptr->my_weapon = patterns::weapons[rand()%20];
		p_ptr->my_other_weapon = patterns::weapons[rand()%20];
		break;
	case 'm':
		p_ptr->my_weapon = patterns::weapons[rand()%100];
		p_ptr->my_other_weapon = patterns::weapons[rand()%100];
		break;
	case 'q':
		p_ptr->my_weapon = patterns::weapons[rand()%16];
		p_ptr->my_other_weapon = patterns::weapons[rand()%16];
		break;
	case 'u':
		p_ptr->my_weapon = patterns::weapons[0];
		p_ptr->my_other_weapon = patterns::weapons[1];
		break;
	case 't':
		p_ptr->my_weapon = patterns::weapons[0];
		p_ptr->my_other_weapon = patterns::weapons[0];
		break;
	}
	
	patterns::note_all(false,-1);
}


void init()
{
	srand((unsigned int)time(NULL));
	initscr();
	int x = 0, y = 0;
	getmaxyx(stdscr,y,x);
	if(y < 30 || x < 80)
		resize_term(std::max(30,y),std::max(x,80));
	getmaxyx(stdscr,y,x);
	if(y < 30 || x < 80)
	{
		endwin();
		printf("Please resize your terminal to at least 80x30.\n");
		getch();
		exit(1);
	}

	if(has_colors() == FALSE)
	{
		endwin();
		printf("Your terminal does not support colour.\n");
		getch();
		exit(1);
	}
    start_color();
	for(int i=0;i<8;i++)
	{
		init_pair(i,i,COLOR_BLACK);
	}
	noecho();
	cbreak();
	curs_set(0);
	keypad(stdscr,TRUE);
}

int main(int argc, char** argv)
{
	bool first_game = true;
	bool restart;
	init();
	do_start_menu();
	do
	{
		restart = false;
		init_game();
		//draw_dungeon();
		//draw_ui();
		//mvaddstr(12,12,"                          ");
		//mvaddstr(13,12," /-----------------------\\ ");
		//mvprintw(14,12," | This game can be very | ");
		//mvprintw(15,12," |  confusing! Read the  | ");
		//mvprintw(16,12," |     readme file!      | ");
		//mvaddstr(17,12," \\-----------------------/ ");
		//mvaddstr(18,12,"                          ");
		//getch();

		for(dungeon_level = 1;dungeon_level!=last_level+1;dungeon_level++)
		{
			if(game_mode == 's' || game_mode == 'm')
			{
				if(dungeon_level == 11 || dungeon_level == 13 || dungeon_level == 17 || dungeon_level == 19)
					dungeon_level++; //making 16 levels
			}
			bosses_generated = 0;
			bosses_killed = 0;
			switch(game_mode)
			{
			case 's': case 'm':
				boss_quota = 11 - dungeon_level/2; break;
			case 'q':
				boss_quota = 5 - dungeon_level/2; break;
			case 'b':
				boss_quota = 6 - dungeon_level/2; break;
			case 't':
				boss_quota = 1; break;
			case 'u':
				boss_quota = 3; break;
			}
			p_ptr->health = player_health_max(dungeon_level);
			p_ptr->max_health = player_health_max(dungeon_level);
			draw_dungeon();
			draw_ui();
			mvaddstr(12,12,"                          ");
			mvaddstr(13,12," /-----------------------\\ ");
			mvprintw(14,12," | Welcome to level %3d!  | ",dungeon_level_number(dungeon_level));
			mvaddstr(15,12," \\-----------------------/ ");
			mvaddstr(16,12,"                          ");
			getch();
			if(game_mode == 't')
				display_tutorial_text();
			do
			{
				if(rand()%3==0)
				{
					if((game_mode != 't') || rand()%2 == 0)
						dungeon::add_monster();
				}
				draw_dungeon();
				draw_ui();
			}
			while(play_turn());
			for(unsigned int i=1;i<actors.size();i++)
				delete actors[i];
			actors.clear();
			if(p_ptr->health <= 0) break;
			if(dungeon_level == last_level) break;
			actors.push_back(p_ptr);
			switch(game_mode)
			{
			case 's': case 'm':
				dungeon::init_dungeon(dungeon_level+((dungeon_level==10||dungeon_level==12||dungeon_level==16||dungeon_level==18)?2:1));
				break;
			case 'b': case 'q': case 'u': case 't':
				dungeon::init_dungeon(dungeon_level+1);
				break;
			}
			while(true)
			{
				int x = rand()%dungeon::MAP_WIDTH;
				int y = rand()%dungeon::MAP_HEIGHT;
				if(dungeon::walkable(x,y))
				{
					p_ptr->x = x;
					p_ptr->y = y;
					dungeon::dungeon_occupants[dungeon::dungeon_index(x,y)] = p_ptr;
					break;
				}
			}
			patterns::note_all(false,-1);
		}
		if(p_ptr->health >0)
		{
			if(game_mode == 't')
			{
				mvaddstr(10,18,"                         ");
				mvaddstr(11,18," /---------------------\\ ");
				mvaddstr(12,18," |  CONGRATULATIONS!   | ");
				mvaddstr(13,18," |    YOU BEAT THE     | ");
				mvaddstr(14,18," |   TUTORIAL AND ARE  | ");
				mvaddstr(15,18," |  NOW READY TO FACE  | ");
				mvaddstr(16,18," |   THE FULL GAME.    | ");
				mvaddstr(17,18," |   Q TO GO TO MENU.  | ");
				mvaddstr(18,18," |     R TO REPLAY.    | ");
				mvaddstr(19,18," \\---------------------/ ");
				mvaddstr(20,18,"                         ");
			}
			else
			{
				mvaddstr(7,18, "                         ");
				mvaddstr(8,18, " /---------------------\\ ");
				mvaddstr(9,18, " |  CONGRATULATIONS!   | ");
				mvaddstr(10,18," |   YOUR MERCILESS    | ");
				mvaddstr(11,18," |   GENOCIDE SAVED    | ");
				mvaddstr(12,18," |   THE HUMAN RA--    | ");
				mvaddstr(13,18," |    ACTUALLY IT      | ");
				mvaddstr(14,18," |     MADE YOU A      | ");
				mvaddstr(15,18," |    WAR CRIMINAL.    | ");
				mvaddstr(16,18," |  F TO WRITE REPORT. | ");
				mvaddstr(17,18," |      Q TO QUIT.     | ");
				mvaddstr(18,18," |    R TO RESTART.    | ");
				mvaddstr(19,18," \\---------------------/ ");
				mvaddstr(20,18,"                         ");
			}
		}
		else
		{
			mvaddstr(12,18,"                       ");
			mvaddstr(13,18," /-------------------\\ ");
			mvaddstr(14,18," |   REST IN PEACE   | ");
			mvaddstr(15,18," |  Q TO GO TO MENU  | ");
			mvaddstr(16,18," |   R TO RESTART.   | ");
			mvaddstr(17,18," |    F TO WRITE     | ");
			mvaddstr(18,18," | ADVENTURE REPORT. | ");
			mvaddstr(19,18," \\-------------------/ ");
			mvaddstr(20,18,"                       ");
		}
		int kp;
		for(kp=getch();!(kp=='q' || kp == 'Q' || kp=='r' || kp == 'R'|| kp=='f' || kp == 'F');kp=getch())
		{
		}
		first_game = false;
		if(kp=='r' || kp == 'R')
			restart = true;
		if(kp=='q'|| kp == 'Q')
		{
			do_start_menu();
			restart = true;
		}
		if(kp=='f'||kp=='F')
		{
			char fname[30];
			mvaddstr(20,20, "Enter filename:");
			move(21,20);
			echo();
			getstr(fname);
			noecho();
			std::ofstream of(fname);
			//of.open//(std::string(fname)/*+".txt"*/);
			of << "You entered the dungeon on turn zero.\n";
			of << "Soon afterwards, on level " << dungeon_level << ", you " << ((p_ptr->health <= 0) ? "died.\n" : "killed the final boss.\n");
			of << "On the way, you killed " << enemies_killed[0] << " 'a's, " << enemies_killed[1] << " 'b's, " << enemies_killed[2] << " 'c's, " << enemies_killed[3] << " 'd's, and " << enemies_killed[4] << " 'E's.\n";
			of << "In return, the 'a's did " << damage_from_enemies[0] << " damage to you, the 'b's did " << damage_from_enemies[1] << ", the 'c's " << damage_from_enemies[2] << ", the 'd's " << damage_from_enemies[3] << " and the 'E's " << damage_from_enemies[4] << ".\n";

			of.close();
		}
	} while (restart);
	return 0;
}